<?php

use CBX\RBooking\CBXRBookingHooks;

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXRBooking
 * @subpackage CBXRBooking/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    CBXRBooking
 * @subpackage CBXRBooking/includes
 * @author     Codeboxr <info@codeboxr.com>
 */
final class CBXRBooking {
	/**
	 * The single instance of the class.
	 *
	 * @var self
	 * @since  2.0.0
	 */
	private static $instance = null;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	protected $hooks;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = CBXRBOOKING_PLUGIN_NAME;
		$this->version     = CBXRBOOKING_PLUGIN_VERSION;

		$this->include_files();
		$this->hooks = new CBXRBookingHooks();
	}//end constructor

	/**
	 * Singleton Instance.
	 *
	 * Ensures only one instance of CBXRBooking is loaded or can be loaded.
	 *
	 * @return self Main instance.
	 * @see run_cbxrbooking()
	 * @since  2.0.0
	 * @static
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}//end method instance

	/**
	 * Cloning is forbidden.
	 *
	 * @since 2.0.0
	 */
	public function __clone() {
		cbxrbooking_doing_it_wrong( __FUNCTION__, esc_html__( 'Cloning is forbidden.', 'cbxrbooking' ), '2.0.0' );
	}//end method clone

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 2.0.0
	 */
	public function __wakeup() {
		cbxrbooking_doing_it_wrong( __FUNCTION__, esc_html__( 'Unserializing instances of this class is forbidden.', 'cbxrbooking' ), '2.0.0' );
	}//end method wakeup

	/**
	 * Include necessary files
	 *
	 * @return void
	 */
	private function include_files() {
		require_once __DIR__ . '/../lib/autoload.php';

		include_once __DIR__ . '/CBXRBookingEmails.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/Widgets/Widget/CBXRBookingWidget.php';
	}//end method include_files
}//end class CBXRBooking