<?php

namespace CBX\RBooking;

use CBX\RBooking\Helpers\CBXRBookingHelper;
use Exception;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXRBooking
 * @subpackage CBXRBooking/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    CBXRBooking
 * @subpackage CBXRBooking/admin
 * @author     Codeboxr <info@codeboxr.com>
 */
class CBXRBookingAdmin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	public $settings;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct() {

		$this->plugin_name = CBXRBOOKING_PLUGIN_NAME;
		$this->version     = CBXRBOOKING_PLUGIN_VERSION;

		//get instance of setting api
		$this->settings = new CBXRBookingSettings();
	}//end constructor method

	// remove date dropdown
	public function remove_date_dropdown() {
		$screen = get_current_screen();

		if ( 'cbxrbooking' === $screen->post_type ) {
			add_filter( 'months_dropdown_results', '__return_empty_array' );
		}
	}//end remove_date_dropdown

	/**
	 * Initialize setting
	 */
	public function setting_init() {
		//set the settings
		$this->settings->set_sections( $this->get_settings_sections() );
		$this->settings->set_fields( $this->get_settings_fields() );
		//initialize settings
		$this->settings->admin_init();
	}//end setting_init

	/**
	 * Global Setting Sections and titles
	 *
	 * @return array
	 */
	public function get_settings_sections() {
		return CBXRBookingHelper::get_settings_sections();
	}//end method get_settings_sections

	/**
	 * Global Setting Fields
	 *
	 * @return array
	 */
	public function get_settings_fields() {
		return CBXRBookingHelper::get_settings_fields();
	}//end method get_settings_fields

	/**
	 * Register Custom Post Type cbxrbooking
	 *
	 * @since    3.7.0
	 */
	public function create_cbxrbooking_post_type() {
		$labels = [
			'name'          => _x( 'Booking Forms', 'Post Type General Name', 'cbxrbooking' ),
			'singular_name' => _x( 'Booking Form', 'Post Type Singular Name', 'cbxrbooking' ),
			'menu_name'     => esc_html__( 'Rest. Booking', 'cbxrbooking' ),
			//'parent_item_colon'  => esc_html__( 'Parent Item:', 'cbxrbooking' ),
			'all_items'     => esc_html__( 'Booking Forms', 'cbxrbooking' ),
			'view_item'     => esc_html__( 'View Booking Form', 'cbxrbooking' ),
			'add_new_item'  => esc_html__( 'Add New Booking Form', 'cbxrbooking' ),
			'add_new'       => esc_html__( 'Add New Form', 'cbxrbooking' ),
			'edit_item'     => esc_html__( 'Edit Booking Form', 'cbxrbooking' ),
			'update_item'   => esc_html__( 'Update Booking Form', 'cbxrbooking' ),
			'search_items'  => esc_html__( 'Search Booking Form', 'cbxrbooking' ),
		];

		$args = [
			'label'               => esc_html__( 'Comfort Restaurant Booking', 'cbxrbooking' ),
			'description'         => esc_html__( 'Comfort Restaurant Booking', 'cbxrbooking' ),
			'labels'              => $labels,
			'supports'            => [ 'title' ],
			'hierarchical'        => false,
			'public'              => false,
			'show_ui'             => true,
			'show_in_menu'        => false, //to hide from dashboard menu
			'show_in_nav_menus'   => false,
			'show_in_admin_bar'   => false,
			'menu_icon'           => plugins_url( 'assets/images/cbxbooking_menu_icon.png?v=3',
				dirname( __FILE__ ) ),
			'can_export'          => true,
			'has_archive'         => false,
			'exclude_from_search' => true,
			'publicly_queryable'  => false,
			'capability_type'     => 'post',
		];
		register_post_type( 'cbxrbooking', $args );
	}

	/**
	 * Register the administration menu for this plugin into the WordPress Dashboard menu.
     *
	 * @since    1.0.0
	 */
	public function admin_menus() {
		global $submenu;

		add_menu_page(
			esc_html__( 'Comfort Restaurant Booking: Form Manager', 'cbxrbooking' ),
			esc_html__( 'Rest. Booking', 'cbxrbooking' ),
			'cbxrbooking_dashboard_manage',
			'cbxrbooking-dashboard',
			[ $this, 'display_admin_dashboard_page' ],
			plugins_url( 'assets/images/cbxbooking_menu_icon.png?v=2', dirname( __FILE__ ) )
		);

		//restaurant log listing
		add_submenu_page(
			'cbxrbooking-dashboard',
			esc_html__( 'Bookings Logs', 'cbxrbooking' ),
			esc_html__( 'Bookings Logs', 'cbxrbooking' ),
			'cbxrbooking_branch_manage',
			'cbxrbookinglogs',
			[ $this, 'display_log_listing_page' ], 1
		);

		add_submenu_page(
			'cbxrbooking-dashboard',
			esc_html__( 'Booking Forms', 'cbxrbooking' ),
			esc_html__( 'Booking Forms', 'cbxrbooking' ),
			'cbxrbooking_form_manage',
			'cbxrbooking-form',
			[ $this, 'display_form_manager_page' ], 3
		);

		add_submenu_page(
			'cbxrbooking-dashboard',
			esc_html__( 'Branch Manager', 'cbxrbooking' ),
			esc_html__( 'Branch Manager', 'cbxrbooking' ),
			'manage_options',
			'cbxrbookingbranchmanager',
			[ $this, 'display_branch_manager_listing_page' ], 4
		);

		//add settings for this plugin
		add_submenu_page(
			'cbxrbooking-dashboard',
			esc_html__( 'Setting', 'cbxrbooking' ),
			esc_html__( 'Setting', 'cbxrbooking' ),
			'cbxrbooking_settings_manage',
			'cbxrbookingsettings',
			[ $this, 'display_plugin_admin_settings' ], 7
		);

		add_submenu_page( 'cbxrbooking-dashboard',
			esc_html__( 'Comfort Restaurant Booking: Email Manager', 'cbxrbooking' ),
			esc_html__( 'Emails', 'cbxrbooking' ),
			'cbxrbooking_settings_manage',
			'cbxrbooking-emails',
			[ $this, 'admin_menu_display_emails' ], 8
		);

		// Tools submenu add
		add_submenu_page(
			'cbxrbooking-dashboard',
			esc_html__( 'Comfort Restaurant Booking: Tools', 'cbxrbooking' ),
			esc_html__( 'Tools', 'cbxrbooking' ),
			'cbxrbooking_settings_manage',
			'cbxrbooking-tools',
			[ $this, 'display_tools_submenu_page' ], 9
		);

		//add help & support page for this plugin
		add_submenu_page(
			'cbxrbooking-dashboard',
			esc_html__( 'Helps & Updates', 'cbxrbooking' ),
			esc_html__( 'Helps & Updates', 'cbxrbooking' ),
			'cbxrbooking_settings_manage',
			'cbxrbooking-help-support',
			[ $this, 'display_plugin_help_support' ], 10
		);

		if ( isset( $submenu['cbxrbooking-dashboard'][0][0] ) ) {
			$submenu['cbxrbooking-dashboard'][0][0] = esc_html__( 'Dashboard', 'cbxrbooking' );
		}

		do_action( 'cbxrbooking_admin_menus_end' );

	}//end method admin_menus

	/**
	 * Show admin dashboard
	 *
	 * @return void
	 */
	public function display_admin_dashboard_page(): void {
		$settings = $this->settings;
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo cbxrbooking_get_template_html( 'admin/dashboard-global.php', [ 'settings' => $settings ] );
	}//end method display_admin_dashboard_page

	/**
	 * Display settings
	 * @global type $wpdb
	 */
	public function display_plugin_admin_settings() {
		global $wpdb;
		$plugin_basename = plugin_basename( plugin_dir_path( __DIR__ ) . CBXRBOOKING_PLUGIN_NAME . '.php' );
		$plugin_data     = get_plugin_data( plugin_dir_path( __DIR__ ) . '/../' . $plugin_basename );

		//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo cbxrbooking_get_template_html( 'admin/settings.php', [
			'plugin_data' => $plugin_data,
			'ref'         => $this,
			'settings'    => $this->settings
		] );
	}

	/**
	 * Loads emails menu template
	 *
	 * @since 2.0.0
	 */
	public function admin_menu_display_emails() {
		$settings = $this->settings;

		$mail_helper = cbxrbooking_mailer();
		$emails      = $mail_helper->emails;

		$template_data = [ 'settings' => $settings, 'emails' => $emails, 'edit' => 0 ];

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		if ( isset( $_REQUEST['edit'] ) && $_REQUEST['edit'] != '' ) {
			$email_id              = sanitize_text_field( wp_unslash( $_REQUEST['edit'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$template_data['edit'] = 1;
			$template_data['id']   = $email_id;
		}

		echo cbxrbooking_get_template_html( 'admin/email_manager.php', $template_data );//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}//end method admin_menu_display_emails

	/**
	 * Display help & support
	 * @global type $wpdb
	 */
	public function display_plugin_help_support() {
		global $wpdb;
		echo cbxrbooking_get_template( 'admin/support.php' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}// end method display_plugin_help_support

	/**
	 * Display admin form page
	 */
	public function display_form_manager_page() {
		global $wpdb;
		echo cbxrbooking_get_template( 'admin/forms.php' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}// end method display_form_manager_page

	/**
	 * Restaurant log listing page
	 */
	public function display_log_listing_page() {
		echo cbxrbooking_get_template( 'admin/admin-booking-logs.php' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}// end method display_log_listing_page

	/**
	 * Show application tools admin
	 *
	 * @return void
	 */
	public function display_tools_submenu_page() {
		echo cbxrbooking_get_template( 'admin/tools.php' ); //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}//end method display_tools_submenu_page

	public function cbxrbooking_log_results_columns( $columns = [] ) {
		$columns = [
			'cb'           => '<input type="checkbox" />', //Render a checkbox instead of text
			'id'           => esc_html__( 'ID', 'cbxrbooking' ),
			'form_id'      => esc_html__( 'Form', 'cbxrbooking' ),
			'branch_id'    => esc_html__( 'Branch', 'cbxrbooking' ),
			'secret'       => esc_html__( 'Code', 'cbxrbooking' ),
			'booking_date' => esc_html__( 'Date', 'cbxrbooking' ),
			'booking_time' => esc_html__( 'Time', 'cbxrbooking' ),
			'party_size'   => esc_html__( 'Size', 'cbxrbooking' ),
			'user_name'    => esc_html__( 'Name', 'cbxrbooking' ),
			'user_email'   => esc_html__( 'Email', 'cbxrbooking' ),
			'user_phone'   => esc_html__( 'Phone', 'cbxrbooking' ),
			'user_message' => esc_html__( 'Message', 'cbxrbooking' ),
			'user_ip'      => esc_html__( 'IP', 'cbxrbooking' ),
			'status'       => esc_html__( 'Status', 'cbxrbooking' )
		];

		return $columns;
	}

	/**
	 * Meta form settings section for CBXForm
	 * @return array
	 */
	public function cbxrbookingform_meta_settings_sections() {
		$sections = [
			[
				'id'    => 'cbxrbooking_style',
				'title' => esc_html__( 'General', 'cbxrbooking' )
			],
			[
				'id'    => 'cbxrbooking_booking_schedule',
				'title' => esc_html__( 'Schedule', 'cbxrbooking' )
			],
			[
				'id'    => 'cbxrbooking_email_admin',
				'title' => esc_html__( 'Admin Alert', 'cbxrbooking' )
			],
			[
				'id'    => 'cbxrbooking_email_user',
				'title' => esc_html__( 'User Alert', 'cbxrbooking' )
			],
			[
				'id'    => 'cbxrbooking_cancel_booking',
				'title' => esc_html__( 'Booking Cancel', 'cbxrbooking' )
			],
			[
				'id'    => 'cbxrbooking_misc',
				'title' => esc_html__( 'Misc', 'cbxrbooking' )
			],
			// [
			// 	'id'    => 'cbxrbooking_integration',
			// 	'title' => esc_html__( 'Integration', 'cbxrbooking' )
			// ]
		];


		return $sections;
	}//end cbxrbookingform_meta_settings_sections


	/**
	 * Get the default state dropdown
	 *
	 * @return array
	 */
	public function get_default_state() {
		return CBXRBookingHelper::get_default_states();
	}


	/**
	 * Get the early bookings dropdown
	 *
	 * @return array
	 */
	public function get_early_bookings() {
		return CBXRBookingHelper::get_early_bookings();
	}

	/**
	 * Get the late bookings dropdown
	 *
	 * @return array
	 */
	public function get_late_bookings() {
		return CBXRBookingHelper::get_late_bookings();
	}

	/**
	 * Get the time dropdown
	 *
	 * @return array
	 */
	public function get_time_interval() {
		return CBXRBookingHelper::get_time_interval();
	}

	/**
	 * Get the week starts on dropdown
	 *
	 * @return array
	 */
	public function get_week_starts_on() {
		return CBXRBookingHelper::get_week_starts_on();
	}

	/**
	 * Get the early bookings cancel dropdown
	 *
	 * @return array
	 */
	public function get_early_cancels() {
		return CBXRBookingHelper::get_early_cancels();
	}

	/**
	 * Determines whether or not the current user has the ability to save meta data associated with this post.
	 *
	 * @param int $post_id The ID of the post being save
	 * @param            $action
	 * @param            $nonce
	 *
	 * @return bool
	 */
	public function user_can_save( $post_id, $action, $nonce ) {
		$is_autosave    = wp_is_post_autosave( $post_id );
		$is_revision    = wp_is_post_revision( $post_id );
		$is_valid_nonce = ( isset( $nonce ) && wp_verify_nonce( $nonce, $action ) );

		// Return true if the user is able to save; otherwise, false.
		return ! ( $is_autosave || $is_revision ) && $is_valid_nonce;
	}

	/**
	 * add branch dropdown filter
	 */
	public function admin_posts_filter_add_branch_dropdown() {
		$post_type = 'cbxrbooking';
		if ( isset( $_GET['post_type'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
			$post_type = sanitize_text_field( wp_unslash( $_GET['post_type'])); // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		}

		//add filter to the post type you want
		if ( 'cbxrbooking' === $post_type ) {
			global $wpdb;
			$branch_table = $wpdb->prefix . 'cbxrbooking_branch_manager';
			// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
			$all_branches = $wpdb->get_results( "SELECT id, name FROM $branch_table", 'ARRAY_A' );
			$branches_kv  = [];
			foreach ( $all_branches as $single_branch ) {
				$branches_kv[ $single_branch['name'] ] = $single_branch['id'];
			}

			?>
            <select name="branch_id">
                <option value=""><?php esc_html_e( 'All Branch', 'cbxrbooking' ); ?></option>
				<?php
				$current_branch = isset( $_GET['branch_id'] ) ? intval( $_GET['branch_id'] ) : 0; // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
				foreach ( $branches_kv as $label => $value ) {
					printf(
						'<option value="%s"%s>%s</option>',
						esc_attr($value),
						$value == $current_branch ? ' selected="selected"' : '',
						esc_html($label)
					);
				}
				?>
            </select>
		<?php }
	}//end admin_posts_filter_add_branch_dropdown

	/**
	 * add branch filter to query
	 *
	 * @param $query
	 */
	function admin_posts_parse_query_modification( $query ) {
		global $pagenow;

		if ( isset( $_GET['post_type'] ) && $_GET['post_type'] === 'cbxrbooking' && isset( $_GET['branch_id'] ) && intval( $_GET['branch_id'] ) > 0 ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
			$query->query_vars['meta_key']   = '_cbxrbookingformmeta_branch'; // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_key
			$query->query_vars['meta_value'] = intval( $_GET['branch_id'] ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.DB.SlowDBQuery.slow_db_query_meta_value
		}
	}//end admin_posts_parse_query_modification

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles( $hook ) {

		global $post_type;
		$page = isset( $_GET['page'] ) ? sanitize_text_field( wp_unslash( $_GET['page'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		$ver  = $this->version;

		$vendors_url_part = CBXRBOOKING_ROOT_URL . 'assets/vendors/';
		$css_url_part     = CBXRBOOKING_ROOT_URL . 'assets/css/';

		wp_register_style( 'select2', plugin_dir_url( __FILE__ ) . '../assets/vendors/select2/css/select2.min.css',
			[],
			$ver );

		wp_register_style( 'awesome-notifications', $vendors_url_part . 'awesome-notifications/style.css', [],
			$ver );

		wp_register_style( 'flatpickr', plugin_dir_url( __FILE__ ) . '../assets/vendors/flatpickr/flatpickr.min.css',
			[], $ver, 'all' );

		wp_register_style( 'pickr', $vendors_url_part . 'pickr/classic.min.css', [], $ver );

		wp_register_style( 'cbxrbooking-settings',
			plugin_dir_url( __FILE__ ) . '../assets/css/cbxrbooking-setting.css',
			[ 'select2', 'awesome-notifications', 'pickr' ], $ver, 'all' );

		wp_register_style( 'cbxrbooking-admin', plugin_dir_url( __FILE__ ) . '../assets/css/cbxrbooking-admin.css',
			[], $ver, 'all' );

		wp_register_style( 'cbxrbooking-builder', plugin_dir_url( __FILE__ ) . '../assets/css/cbxrbooking-builder.css',
			[], $ver, 'all' );

		wp_register_style( 'cbxrbooking-dashboard', $css_url_part . 'cbxrbooking-dashboard.css', [], $ver, 'all' );

		//for setting page
		if ( $page == 'cbxrbookingsettings' ) {
			wp_enqueue_media();

			wp_enqueue_style( 'select2' );
			wp_enqueue_style( 'awesome-notifications' );
			wp_enqueue_style( 'pickr' );
			wp_enqueue_style( 'cbxrbooking-settings' );
			wp_enqueue_style( 'cbxrbooking-admin' );
		}

		if ( $page == 'cbxrbookinglogs' || $page == 'cbxrbookingbranchmanager' || $page == 'cbxrbooking-form' ) {
			wp_enqueue_style( 'cbxrbooking-builder' );
			wp_enqueue_style( 'cbxrbooking-admin' );
		}

		if ( $page == 'cbxrbooking-tools' || $page == 'cbxrbooking-dashboard' ) {
			wp_enqueue_style( 'cbxrbooking-admin' );
			wp_enqueue_style( 'cbxrbooking-dashboard' );
		}

		if ( $page == 'cbxrbooking-help-support' ) {
			wp_enqueue_style( 'cbxrbooking-admin' );
		}

		wp_register_style( 'cbxrbooking-email-manager', plugin_dir_url( __FILE__ ) . '../assets/css/cbxrbooking-email-manager.css', [], $this->version, 'all' );

		if ( $page == 'cbxrbooking-emails' ) {
			wp_enqueue_style( 'cbxrbooking-email-manager' );
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts( $hook ) {
		$page = isset( $_GET['page'] ) ? sanitize_text_field( wp_unslash( $_GET['page'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Recommended 

		$ver    = $this->version;
		$footer = [
			'in_footer' => true,
		];

		$vendors_url_part  = CBXRBOOKING_ROOT_URL . 'assets/vendors/';
		$js_url_part_build = CBXRBOOKING_ROOT_URL . 'assets/js/build/';


		wp_register_script( 'pickr', $vendors_url_part . 'pickr/pickr.min.js', [], $ver, true );
		wp_register_script( 'awesome-notifications', $vendors_url_part . 'awesome-notifications/script.js', [],
			$ver, $footer );

		wp_register_script( 'select2', plugin_dir_url( __FILE__ ) . '../assets/vendors/select2/js/select2.min.js',
			[ 'jquery' ], $ver, true );

		wp_register_script( 'cbxrbooking-settings',
			plugin_dir_url( __FILE__ ) . '../assets/js/cbxrbooking-settings.js',
			[ 'jquery', 'select2', 'pickr', 'awesome-notifications' ],
			$ver, true );

		$cbxrbooking_setting_js_vars = apply_filters(
			'cbxrbooking_setting_js_vars',
			[
				'ajaxurl'                  => admin_url( 'admin-ajax.php' ),
				'ajax_fail'                => esc_html__( 'Request failed, please reload the page.',
					'cbxrbooking' ),
				'nonce'                    => wp_create_nonce( 'cbxrbookingnonce' ),
				'is_user_logged_in'        => is_user_logged_in() ? 1 : 0,
				'please_select'            => esc_html__( 'Please Select', 'cbxrbooking' ),
				'search'                   => esc_html__( 'Search...', 'cbxrbooking' ),
				'upload_title'             => esc_html__( 'Window Title', 'cbxrbooking' ),
				'search_placeholder'       => esc_html__( 'Search here', 'cbxrbooking' ),
				'teeny_setting'            => [
					'teeny'         => true,
					'media_buttons' => true,
					'editor_class'  => '',
					'textarea_rows' => 5,
					'quicktags'     => false,
					'menubar'       => false,
				],
				'copycmds'                 => [
					'copy'       => esc_html__( 'Copy', 'cbxrbooking' ),
					'copied'     => esc_html__( 'Copied', 'cbxrbooking' ),
					'copy_tip'   => esc_html__( 'Click to copy', 'cbxrbooking' ),
					'copied_tip' => esc_html__( 'Copied to clipboard', 'cbxrbooking' ),
				],
				'confirm_msg'              => esc_html__( 'Are you sure to remove this step?',
					'cbxrbooking' ),
				'confirm_msg_all'          => esc_html__( 'Are you sure to remove all steps?',
					'cbxrbooking' ),
				'confirm_yes'              => esc_html__( 'Yes', 'cbxrbooking' ),
				'confirm_no'               => esc_html__( 'No', 'cbxrbooking' ),
				'are_you_sure_global'      => esc_html__( 'Are you sure?', 'cbxrbooking' ),
				'are_you_sure_delete_desc' => esc_html__( 'Once you delete, it\'s gone forever. You can not revert it back.',
					'cbxrbooking' ),
				'pickr_i18n'               => [
					// Strings visible in the UI
					'ui:dialog'       => esc_html__( 'color picker dialog', 'cbxrbooking' ),
					'btn:toggle'      => esc_html__( 'toggle color picker dialog', 'cbxrbooking' ),
					'btn:swatch'      => esc_html__( 'color swatch', 'cbxrbooking' ),
					'btn:last-color'  => esc_html__( 'use previous color', 'cbxrbooking' ),
					'btn:save'        => esc_html__( 'Save', 'cbxrbooking' ),
					'btn:cancel'      => esc_html__( 'Cancel', 'cbxrbooking' ),
					'btn:clear'       => esc_html__( 'Clear', 'cbxrbooking' ),

					// Strings used for aria-labels
					'aria:btn:save'   => esc_html__( 'save and close', 'cbxrbooking' ),
					'aria:btn:cancel' => esc_html__( 'cancel and close', 'cbxrbooking' ),
					'aria:btn:clear'  => esc_html__( 'clear and close', 'cbxrbooking' ),
					'aria:input'      => esc_html__( 'color input field', 'cbxrbooking' ),
					'aria:palette'    => esc_html__( 'color selection area', 'cbxrbooking' ),
					'aria:hue'        => esc_html__( 'hue selection slider', 'cbxrbooking' ),
					'aria:opacity'    => esc_html__( 'selection slider', 'cbxrbooking' ),
				],
				'awn_options'              => [
					'tip'           => esc_html__( 'Tip', 'cbxrbooking' ),
					'info'          => esc_html__( 'Info', 'cbxrbooking' ),
					'success'       => esc_html__( 'Success', 'cbxrbooking' ),
					'warning'       => esc_html__( 'Attention', 'cbxrbooking' ),
					'alert'         => esc_html__( 'Error', 'cbxrbooking' ),
					'async'         => esc_html__( 'Loading', 'cbxrbooking' ),
					'confirm'       => esc_html__( 'Confirmation', 'cbxrbooking' ),
					'confirmOk'     => esc_html__( 'OK', 'cbxrbooking' ),
					'confirmCancel' => esc_html__( 'Cancel', 'cbxrbooking' )
				],
				'global_setting_link_html' => '<a href="' . admin_url( 'admin.php?page=cbxrbooking-settings' ) . '"  class="button outline primary pull-right">' . esc_html__( 'Global Settings',
						'cbxrbooking' ) . '</a>',
				'lang'                     => get_user_locale(),
			]
		);

		wp_localize_script( 'cbxrbooking-settings', 'cbxrbooking_setting',
			apply_filters( 'cbxrbooking_setting_js_vars', $cbxrbooking_setting_js_vars ) );


		//for setting page
		if ( $page == 'cbxrbookingsettings' ) {
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'select2' );
			wp_enqueue_script( 'pickr' );
			wp_enqueue_script( 'cbxrbooking-settings' );
		}

		if ( $page == 'cbxrbookinglogs' ) {

			$js_translations = CBXRBookingHelper::cbxrbooking_log_builder_js_translation();

			if ( defined( 'CBXRBOOKING_DEV_MODE' ) && CBXRBOOKING_DEV_MODE == true ) {
				//for development version
				wp_register_script( 'cbxrbooking_log_vue_dev',
					'http://localhost:8880/assets/vuejs/apps/admin/cbxrbookinglog.js', [], $ver, true );
				wp_localize_script( 'cbxrbooking_log_vue_dev', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_log_vue_dev' );
			} else {
				// for production
				wp_register_script( 'cbxrbooking_log_vue_main', $js_url_part_build . 'cbxrbookinglog.js',
					[], $ver, true );
				wp_localize_script( 'cbxrbooking_log_vue_main', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_log_vue_main' );
			}

		}

		if ( $page == 'cbxrbookingbranchmanager' ) {

			$js_translations = CBXRBookingHelper::cbxrbooking_branch_builder_js_translation();

			if ( defined( 'CBXRBOOKING_DEV_MODE' ) && CBXRBOOKING_DEV_MODE == true ) {
				//for development version
				wp_register_script( 'cbxrbooking_branch_vue_dev',
					'http://localhost:8880/assets/vuejs/apps/admin/cbxrbookingbranch.js', [], $ver, true );
				wp_localize_script( 'cbxrbooking_branch_vue_dev', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_branch_vue_dev' );
			} else {
				// for production
				wp_register_script( 'cbxrbooking_branch_vue_main', $js_url_part_build . 'cbxrbookingbranch.js',
					[], $ver, true );
				wp_localize_script( 'cbxrbooking_branch_vue_main', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_branch_vue_main' );
			}

		}

		if ( $page == 'cbxrbooking-form' ) {

			$js_translations = CBXRBookingHelper::cbxrbooking_form_builder_js_translation();

			if ( defined( 'CBXRBOOKING_DEV_MODE' ) && CBXRBOOKING_DEV_MODE == true ) {
				//for development version
				wp_register_script( 'cbxrbooking_form_vue_dev',
					'http://localhost:8880/assets/vuejs/apps/admin/cbxrbookingform.js', [], $ver, true );
				wp_localize_script( 'cbxrbooking_form_vue_dev', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_form_vue_dev' );
			} else {
				// for production
				wp_register_script( 'cbxrbooking_form_vue_main', $js_url_part_build . 'cbxrbookingform.js',
					[], $ver, true );
				wp_localize_script( 'cbxrbooking_form_vue_main', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_form_vue_main' );
			}

		}

		if ( $page == 'cbxrbooking-tools' ) {
			$js_translations = CBXRBookingHelper::cbxrbooking_tools_js_translation();

			$js_translations['migration_files']      = MigrationManage::migration_files();
			$js_translations['migration_files_left'] = MigrationManage::migration_files_left();

			if ( defined( 'CBXRBOOKING_DEV_MODE' ) && CBXRBOOKING_DEV_MODE == true ) {
				//for development version
				wp_register_script( 'cbxrbooking_tools_vue_dev',
					'http://localhost:8880/assets/vuejs/apps/admin/cbxrbookingtools.js', [], $ver, true );
				wp_localize_script( 'cbxrbooking_tools_vue_dev', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_tools_vue_dev' );
			} else {
				// for production
				wp_register_script( 'cbxrbooking_tools_vue_main', $js_url_part_build . 'cbxrbookingtools.js',
					[], $ver, true );
				wp_localize_script( 'cbxrbooking_tools_vue_main', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_tools_vue_main' );
			}
		}

		if ( $page == 'cbxrbooking-dashboard' ) {
			$js_translations = CBXRBookingHelper::cbxrbooking_dashboard_js_translation();


			if ( defined( 'CBXRBOOKING_DEV_MODE' ) && CBXRBOOKING_DEV_MODE == true ) {
				//for development version
				wp_register_script( 'cbxrbooking_dashboard_vue_dev',
					'http://localhost:8880/assets/vuejs/apps/admin/cbxrbookingdashboard.js', [], $ver, true );
				wp_localize_script( 'cbxrbooking_dashboard_vue_dev', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_dashboard_vue_dev' );
			} else {
				// for production
				wp_register_script( 'cbxrbooking_dashboard_vue_main', $js_url_part_build . 'cbxrbookingdashboard.js',
					[], $ver, true );
				wp_localize_script( 'cbxrbooking_dashboard_vue_main', 'cbxrbooking_vue_var', $js_translations );
				wp_enqueue_script( 'cbxrbooking_dashboard_vue_main' );
			}
		}
	}

	/**
	 * show branch manager listing
	 */
	public function display_branch_manager_listing_page() {
		echo cbxrbooking_get_template( 'admin/branch-manager-list.php' ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	}

	/**
	 * Filters the array of row meta for each/specific plugin in the Plugins list table.
	 * Appends additional links below each/specific plugin on the plugins page.
	 *
	 * @access  public
	 *
	 * @param array $links_array An array of the plugin's metadata
	 * @param string $plugin_file_name Path to the plugin file
	 * @param array $plugin_data An array of plugin data
	 * @param string $status Status of the plugin
	 *
	 * @return  array       $links_array
	 */
	public function plugin_row_meta( $links_array, $plugin_file_name, $plugin_data, $status ) {
		if ( strpos( $plugin_file_name, CBXRBOOKING_BASE_NAME ) !== false ) {
			if ( ! function_exists( 'is_plugin_active' ) ) {
				include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			}

			$links_array[] = '<a target="_blank" style="color:#9c27b0 !important; font-weight: bold;" href="https://wordpress.org/support/plugin/cbxrbooking/" aria-label="' . esc_attr__( 'Free Support',
					'cbxrbooking' ) . '">' . esc_html__( 'Free Support', 'cbxrbooking' ) . '</a>';

			$links_array[] = '<a target="_blank" style="color:#9c27b0 !important; font-weight: bold;" href="https://wordpress.org/plugins/cbxrbooking/#reviews" aria-label="' . esc_attr__( 'Reviews',
					'cbxrbooking' ) . '">' . esc_html__( 'Reviews', 'cbxrbooking' ) . '</a>';

			$links_array[] = '<a target="_blank" style="color:#9c27b0 !important; font-weight: bold;" href="https://codeboxr.com/doc/cbxrbooking-doc/" aria-label="' . esc_attr__( 'Documentation',
					'cbxrbooking' ) . '">' . esc_html__( 'Documentation', 'cbxrbooking' ) . '</a>';


			if ( in_array( 'cbxrbookingpro/cbxrbookingpro.php', apply_filters( 'active_plugins',
					get_option( 'active_plugins' ) ) ) || defined( 'CBXRBOOKINGPRO_PLUGIN_NAME' ) ) {

			} else {
				$links_array[] = '<a target="_blank" style="color:#9c27b0 !important; font-weight: bold;" href="https://codeboxr.com/product/comfort-restaurant-booking-for-wordpress/#downloadarea" aria-label="' . esc_attr__( 'Try Pro Addon',
						'cbxrbooking' ) . '">' . esc_html__( 'Try Pro Addon', 'cbxrbooking' ) . '</a>';
			}
		}

		return $links_array;
	}//end plugin_row_meta

	/**
	 * Show action links on the plugin screen.
	 *
	 * @param mixed $links Plugin Action links.
	 *
	 * @return  array
	 */
	public static function plugin_action_links( $links ) {

		$action_links = [
			'settings' => '<a style="color: #2153cc !important; font-weight: bold;" href="' . admin_url( 'admin.php?page=cbxrbookingsettings' ) . '" aria-label="' . esc_attr__( 'View settings',
					'cbxrbooking' ) . '">' . esc_html__( 'Settings', 'cbxrbooking' ) . '</a>',
		];

		return array_merge( $action_links, $links );
	}//end plugin_action_links

	/**
	 * If we need to do something in upgrader process is completed for poll plugin
	 *
	 * @param $upgrader_object
	 * @param $options
	 */
	public function plugin_upgrader_process_complete( $upgrader_object, $options ) {
		$saved_version = get_option( 'cbxrbooking_version' );

		if ( $saved_version === false || version_compare( $saved_version, CBXRBOOKING_PLUGIN_VERSION, '<' ) ) {

			add_action( 'init', [ $this, 'plugin_upgrader_process_complete_partial' ] );

			set_transient( 'cbxrbooking_upgraded_notice', 1 );
			update_option( 'cbxrbooking_version', CBXRBOOKING_PLUGIN_VERSION );
		}
	}//end plugin_upgrader_process_complete

	/**
	 * Plugin upgrader partial functions
	 *
	 * @return void
	 * @since 2.0.0
	 */
	public function plugin_upgrader_process_complete_partial() {
		CBXRBookingHelper::create_table();
	}//end method plugin_upgrader_process_complete_partial

	/**
	 * Show a notice to anyone who has just installed the plugin for the first time
	 * This notice shouldn't display to anyone who has just updated this plugin
	 */
	public function plugin_activate_upgrade_notices() {
		// Check the transient to see if we've just activated the plugin
		if ( get_transient( 'cbxrbooking_activated_notice' ) ) {
			echo '<div class="notice notice-success is-dismissible">';
			/* translators: %s: Version */
			echo '<p>' . sprintf( __( 'Thanks for installing/deactivating <strong>Comfort Restaurant Booking</strong> V%s - Codeboxr Team', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					'cbxrbooking' ), esc_attr(CBXRBOOKING_PLUGIN_VERSION )) . '</p>';
			/* translators: %1$s: settings url , %2$s: plugin url  */
			echo '<p>' . sprintf( __( 'Check <a href="%1$s">Plugin Setting</a> | <a href="%2$s" target="_blank">Learn More</a>', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					'cbxrbooking' ), esc_url(admin_url( 'admin.php?page=cbxrbookingsettings' )),
					esc_url('https://codeboxr.com/product/comfort-restaurant-booking-for-wordpress/' )) . '</p>';
			echo '</div>';
			// Delete the transient so we don't keep displaying the activation message
			delete_transient( 'cbxrbooking_activated_notice' );

			$this->pro_addon_compatibility_campaign();
		}

		// Check the transient to see if we've just activated the plugin
		if ( get_transient( 'cbxrbooking_upgraded_notice' ) ) {
			echo '<div class="notice notice-success is-dismissible">';
			/* translators: %s: Version */
			echo '<p>' . sprintf( __( 'Thanks for upgrading <strong>Comfort Restaurant Booking</strong> V%s , enjoy the new features and bug fixes - Codeboxr Team', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					'cbxrbooking' ), esc_attr(CBXRBOOKING_PLUGIN_VERSION )) . '</p>';
			/* translators: %1$s: settings url , %2$s: plugin url  */
			echo '<p>' . sprintf( __( 'Check <a href="%1$s">Plugin Setting</a> | <a href="%2$s" target="_blank">Learn More</a>', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					'cbxrbooking' ), esc_url(admin_url( 'admin.php?page=cbxrbookingsettings' )),
					esc_url('https://codeboxr.com/product/comfort-restaurant-booking-for-wordpress/' )) . '</p>';
			echo '</div>';
			// Delete the transient so we don't keep displaying the activation message
			delete_transient( 'cbxrbooking_upgraded_notice' );

			$this->pro_addon_compatibility_campaign();
		}

		if ( get_transient( 'cbxrbooking_proaddon_deactivated' ) ) {
			echo '<div class="notice notice-success is-dismissible" style="border-color: #6648fe !important;">';

			echo '<p>';
			esc_html_e( 'Current version of Comfort Restaurant Booking Pro Addon is not compatible with core  Comfort Restaurant Booking and  Comfort Restaurant Booking Pro Addon is forced deactivate.', 'cbxrbooking' );

			echo '</p>';
			echo '</div>';
			delete_transient( 'cbxrbooking_proaddon_deactivated' );
		}

		if ( get_transient( 'cbxrbooking_frontendaddon_deactivated' ) ) {
			echo '<div class="notice notice-success is-dismissible" style="border-color: #6648fe !important;">';

			echo '<p>';
			esc_html_e( 'Comfort Restaurant Booking 2.0.0 or later version is not compatible with addon plugin CBX Restaurant Booking Frontend Addon as it\'s been merged with Comfort Restaurant Booking Pro from V2.0.0', 'cbxrbooking' );
			echo '</p>';
			echo '</div>';
			delete_transient( 'cbxrbooking_frontendaddon_deactivated' );
		}
	}//end plugin_activate_upgrade_notices

	/**
	 * Check plugin compatibility and pro addon install campaign
	 */
	public function pro_addon_compatibility_campaign() {
		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		/*//frontend addon is active and installed
		if ( in_array( 'cbxrbookingpro/cbxrbookingpro.php',
				apply_filters( 'active_plugins',
					get_option( 'active_plugins' ) ) ) || defined( 'CBXRBOOKINGPRO_PLUGIN_NAME' ) ) {
			//plugin is activated

			$plugin_version   = CBXRBOOKINGPRO_PLUGIN_VERSION;
			$version_required = '1.1.6';


			if ( version_compare( $plugin_version, $version_required, '<' ) ) {
				echo '<div class="notice notice-success is-dismissible"><p>' . sprintf( esc_html__( 'Comfort Restaurant Booking Pro V%s or any previous version is not compatible with Comfort Restaurant Booking V%s or later. Please update Comfort Restaurant Booking Pro to version V%s or later  - Codeboxr Team',
						'cbxrbooking' ), $plugin_version, CBXRBOOKING_PLUGIN_VERSION,
						$version_required ) . '</p></div>';
			}
		} else {
			echo '<div class="notice notice-success is-dismissible"><p>' . sprintf( __( '<a target="_blank" href="%s">Comfort Restaurant Booking Pro</a> enables frontend management for bookings, try it - Codeboxr Team',
					'cbxrbooking' ),
					'https://codeboxr.com/product/cbx-restaurant-booking-frontend-addon/' ) . '</p></div>';
		}*/

		//if the pro addon is active or installed
		if ( in_array( 'cbxrbookingpro/cbxrbookingpro.php', apply_filters( 'active_plugins',
				get_option( 'active_plugins' ) ) ) || defined( 'CBXRBOOKINGPRO_PLUGIN_NAME' ) ) {
			//plugin is activated

			$plugin_version   = CBXRBOOKINGPRO_PLUGIN_VERSION;
			$version_required = '2.0.0';


			if ( version_compare( $plugin_version, $version_required, '<' ) ) {
				/* translators: %1$s: pro version, %2$s: core version, %3$s: version required  */
				echo '<div class="notice notice-success is-dismissible"><p>' . sprintf( esc_html__( 'Comfort Restaurant Booking Pro V%1$s or any previous version is not compatible with Comfort Restaurant Booking V%2$s or later. Please update Comfort Restaurant Booking Pro to version V%3$s or later  - Codeboxr Team',
						'cbxrbooking' ), esc_attr($plugin_version), esc_attr(CBXRBOOKING_PLUGIN_VERSION),
						esc_attr($version_required) ) . '</p></div>';
			}
		} else {
			/* translators: %s: plugin url  */
			echo '<div class="notice notice-success is-dismissible"><p>' . sprintf( __( '<a target="_blank" href="%s">Comfort Restaurant Booking Pro</a> has lots of pro and extended features, try it - Codeboxr Team', // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					'cbxrbooking' ),
					esc_url('https://codeboxr.com/product/comfort-restaurant-booking-for-wordpress/' )) . '</p></div>';
		}
	}//end pro_addon_compatibility_campaign

	/**
	 *
	 *
	 * @param $post_id
	 *
	 * @return array
	 */
	public function cbxrbookingform_meta_settings_fields_by_id( $post_id ) {
		$settings = $this->settings;

		$post_id = absint( $post_id );


		$time_interval  = $this->get_time_interval();
		$week_starts_on = $this->get_week_starts_on();
		$early_cancels  = $this->get_early_cancels();

		//some default values from global config
		$global_date_format  = $settings->get_option( 'global_date_format', 'cbxrbooking_global', 0 );
		$global_time_format  = $settings->get_option( 'global_time_format', 'cbxrbooking_global', 24 );
		$showform_successful = $settings->get_option( 'showform_successful', 'cbxrbooking_global', 'on' );
		$show_credit         = $settings->get_option( 'show_credit', 'cbxrbooking_global', 'no' );

		$settings_builtin_fields =
			[
				'cbxrbooking_style'            =>
					[
						'min_party_size'   => [
							'name'    => 'min_party_size',
							'label'   => esc_html__( 'Min Party Size', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Select minimum party size for bookings.', 'cbxrbooking' ),
							'type'    => 'number',
							'default' => 1,
							'min'     => 1
						],
						'max_party_size'   => [
							'name'    => 'max_party_size',
							'label'   => esc_html__( 'Max Party Size', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Select maximum party size for bookings.', 'cbxrbooking' ),
							'type'    => 'number',
							'default' => 100,
							'min'     => 1,
							'max'     => 10000,
						],
						'require_name'     => [
							'name'    => 'require_name',
							'label'   => esc_html__( 'Require Name', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Is Name is mandatory to request a booking', 'cbxrbooking' ),
							'type'    => 'select',
							'default' => 'name-yes',
							'options' => [
								'name-no'  => esc_html__( 'No', 'cbxrbooking' ),
								'name-yes' => esc_html__( 'Yes', 'cbxrbooking' ),
							]
						],
						'require_email'    => [
							'name'    => 'require_email',
							'label'   => esc_html__( 'Require Email', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Is Email address is mandatory to request a booking',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => 'email-yes',
							'options' => [
								'email-yes' => esc_html__( 'Yes', 'cbxrbooking' ),
								'email-no'  => esc_html__( 'No', 'cbxrbooking' ),
							]
						],
						'require_phone'    => [
							'name'    => 'require_phone',
							'label'   => esc_html__( 'Require Phone', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Is Phone number is mandatory to request a booking',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => 'phone-no',
							'options' => [
								'phone-no'  => esc_html__( 'No', 'cbxrbooking' ),
								'phone-yes' => esc_html__( 'Yes', 'cbxrbooking' ),
							]
						],
						'success_message'  => [
							'name'    => 'success_message',
							'label'   => esc_html__( 'Success Message', 'cbxrbooking' ),
							'type'    => 'wysiwyg',
							'default' => esc_html__( 'Booking request submitted successfully.', 'cbxrbooking' ),
							'desc'    => ''
						],
						'guest_activation' => [
							'name'    => 'guest_activation',
							'label'   => esc_html__( 'Guest Email Activation', 'cbxrbooking' ),
							'desc'    => wp_kses( __( 'Enable/Disable (To make this feature work need to enable user email notification on and user email template should have the tag syntax <code>{activation_link}</code>)',
								'cbxrbooking' ), [ 'code' => [] ] ),
							'type'    => 'checkbox',
							'default' => '',
						],
						'default_state'    => [
							'name'    => 'default_state',
							'label'   => esc_html__( 'Default Booking Status', 'cbxrbooking' ),
							'desc'    => esc_html__( 'What will be status when a new booking is requested?',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => 'pending',
							'options' => CBXRBookingHelper::get_default_states()
						],
						'banned_email'     => [
							'name'        => 'banned_email',
							'label'       => esc_html__( 'Banned Email', 'cbxrbooking' ),
							'desc'        => esc_html__( 'You can block bookings from specific email addresses. Enter each email address on a separate line.',
								'cbxrbooking' ),
							'type'        => 'text_repeat',
							'default'     => '',
							'placeholder' => esc_html__( 'Email Address', 'cbxrbooking' ),
						],
						'banned_ip'        => [
							'name'    => 'banned_ip',
							'label'   => esc_html__( 'Banned IP', 'cbxrbooking' ),
							'desc'    => esc_html__( '	
You can block bookings from specific IP addresses. Enter each IP address on a separate line. Be aware that many internet providers rotate their IP address assignments, so an IP address may accidentally refer to a different user. Also, if you block an IP address used by a public connection, such as cafe WIFI, a public library, or a university network, you may inadvertantly block several people.',
								'cbxrbooking' ),
							'type'    => 'text_repeat',
							'default' => '',

							'placeholder' => esc_html__( 'IP Address', 'cbxrbooking' ),
						],
						'text_beforeform'  => [
							'name'    => 'text_beforeform',
							'label'   => esc_html__( 'Text Before Form', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Text Appear Before Form.', 'cbxrbooking' ),
							'type'    => 'wysiwyg',
							'default' => ''
						],
						'text_afterform'   => [
							'name'    => 'text_afterform',
							'label'   => esc_html__( 'Text After Form', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Text Appear Before Form.', 'cbxrbooking' ),
							'type'    => 'wysiwyg',
							'default' => ''
						],
						'user_consent'     => [
							'name'    => 'user_consent',
							'label'   => esc_html__( 'Show consent checkbox', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Show user consent checkbox to accept privacy policy. It will show wordpress default privacy policy page link',
								'cbxrbooking' ),
							'type'    => 'checkbox',
							'default' => 'on',
						],
					],
				'cbxrbooking_booking_schedule' =>
					[
						'date_format'          => [
							'name'    => 'date_format',
							'label'   => esc_html__( 'Date Format', 'cbxrbooking' ),
							'desc'    => esc_html__( 'What will be the date format?. This date format will use in this booking form activities like booking entry form, edit, email etc.',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => $global_date_format,
							'options' => CBXRBookingHelper::getAllDateFormat(),
						],
						'time_format'          => [
							'name'    => 'time_format',
							'label'   => esc_html__( 'Time Format', 'cbxrbooking' ),
							'desc'    => esc_html__( 'What will be the hour format? 12 or 24. This time format will use in this booking form activities like booking entry form, edit, email etc.',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => $global_time_format,
							'options' => [
								'12' => esc_html__( '12 Hour', 'cbxrbooking' ),
								'24' => esc_html__( '24 Hour', 'cbxrbooking' ),
							],
						],
						'schedule_event'       => [
							'name'     => 'schedule_event',
							'label'    => esc_html__( 'Week Days Schedule', 'cbxrbooking' ),
							'desc'     => wp_kses(__( 'Define the weekly schedule during which you accept bookings. If there is day duplication/repeat then the schedule comes first in order of top to bottom gets the higher priority. <strong>Please note that, schedule added last will get the higher priority</strong>',
								'cbxrbooking' ), ['strong' => []]),
							'type'     => 'scheduler',
							'default'  => '',
							'weekdays' => [
								'1' => _x( 'Mon', 'Monday abbreviation', 'cbxrbooking' ),
								'2' => _x( 'Tue', 'Tuesday abbreviation', 'cbxrbooking' ),
								'3' => _x( 'Wed', 'Wednesday abbreviation', 'cbxrbooking' ),
								'4' => _x( 'Thu', 'Thursday abbreviation', 'cbxrbooking' ),
								'5' => _x( 'Fri', 'Friday abbreviation', 'cbxrbooking' ),
								'6' => _x( 'Sat', 'Saturday abbreviation', 'cbxrbooking' ),
								'0' => _x( 'Sun', 'Sunday abbreviation', 'cbxrbooking' )
							],
						],
						'scheduler_exceptions' => [
							'name'    => 'scheduler_exceptions',
							'label'   => esc_html__( 'Specific Day Exceptions', 'cbxrbooking' ),
							'desc'    => wp_kses(__( 'Define special opening hours for holidays, events or other needs. Leave the time empty if you\'re closed all day.<strong>Please note that, schedule added last will get the higher priority</strong>',
								'cbxrbooking' ), ['strong' => []]),
							'type'    => 'scheduler_exceptions',
							'default' => '',
						],
						'early_bookings'       => [
							'name'    => 'early_bookings',
							'label'   => esc_html__( 'Early Bookings', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Select how early customers can make their booking.',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => '',
							'options' => CBXRBookingHelper::get_early_bookings()
						],
						'late_bookings'        => [
							'name'    => 'late_bookings',
							'label'   => esc_html__( 'Late Bookings', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Select how late customers can make their booking.',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => '',
							'options' => CBXRBookingHelper::get_late_bookings()
						],
						'time_interval'        => [
							'name'    => 'time_interval',
							'label'   => esc_html__( 'Time Interval', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Select the number of minutes between each available time.',
								'cbxrbooking' ),
							'type'    => 'select',
							'default' => 30,
							'options' => $time_interval
						],
						'week_starts_on'       => [
							'name'    => 'week_starts_on',
							'label'   => esc_html__( 'Week Starts On', 'cbxrbooking' ),
							'desc'    => esc_html__( 'Select the first day of the week.', 'cbxrbooking' ),
							'type'    => 'select',
							'default' => '0',
							'options' => $week_starts_on
						],
					],
				'cbxrbooking_email_admin'      => [
					'status' => [
						'name'    => 'status',
						'label'   => esc_html__( 'On/Off', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Status of Email', 'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => 'on',
						//'desc_tip' => true,
					],
					'format' => [
						'name'    => 'format',
						'label'   => esc_html__( 'E-mail Format', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Select the format of the E-mail.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => 'html',
						'options' => [
							'html'  => esc_html__( 'HTML', 'cbxrbooking' ),
							'plain' => esc_html__( 'Plain', 'cbxrbooking' )
						]
					],

					'to'       => [
						'name'           => 'to',
						'label'          => esc_html__( 'To Email', 'cbxrbooking' ),
						'desc'           => esc_html__( 'To Email Address.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => get_bloginfo( 'admin_email' ),
						'desc_tip'       => true,
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'reply_to' => [
						'name'           => 'reply_to',
						'label'          => esc_html__( 'Reply To', 'cbxrbooking' ),
						'desc'           => wp_kses(__( 'Reply To Email Address. Syntax available - <code>{user_email}</code>',
							'cbxrbooking' ), ['code' => []]),
						'type'           => 'text',
						'default'        => '{user_email}',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ]
					],
					'subject'  => [
						'name'           => 'subject',
						'label'          => esc_html__( 'Subject', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email Subject.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'New Booking Request Notification', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => []
					],
					'heading'  => [
						'name'           => 'heading',
						'label'          => esc_html__( 'Heading', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email Template heading.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'New Booking Request', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => []
					],
					'cc'       => [
						'name'           => 'cc',
						'label'          => esc_html__( 'CC', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email CC, for multiple use comma.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => '',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ]
					],
					'bcc'      => [
						'name'           => 'bcc',
						'label'          => esc_html__( 'BCC', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email BCC, for multiple use comma', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => '',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					]
				],
				'cbxrbooking_email_user'       => [
					'status'                             => [
						'name'    => 'status',
						'label'   => esc_html__( 'On/Off', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Status of Email', 'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => 'on'
					],
					'format'                             => [
						'name'    => 'format',
						'label'   => esc_html__( 'E-mail Format', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Select the format of the E-mail.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => 'html',
						'options' => [
							'html'  => esc_html__( 'HTML', 'cbxrbooking' ),
							'plain' => esc_html__( 'Plain', 'cbxrbooking' )
						]
					],
					'reply_to'                           => [
						'name'           => 'reply_to',
						'label'          => esc_html__( 'Reply To', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Reply To Email Address.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => get_bloginfo( 'admin_email' ),
						'desc_tip'       => true,
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'new_booking_user_email_alert'       => [
						'name'    => 'new_booking_user_email_alert',
						'label'   => esc_html__( 'New Booking User Email Alert', 'cbxrbooking' ),
						'desc'    => esc_html__( 'User gets email for new booking request', 'cbxrbooking' ),
						'type'    => 'title',
						'default' => ''
					],
					'new_subject'                        => [
						'name'           => 'new_subject',
						'label'          => esc_html__( 'New Request Email Subject', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email subject user will receive when they make an initial booking request.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'New Booking Request Notification', 'cbxrbooking' ),
						'desc_tip'       => true,
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'new_heading'                        => [
						'name'           => 'new_heading',
						'label'          => esc_html__( 'New Request Email Heading', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email heading user will receive when they make an initial booking request.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'New Booking Request', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'confirmed_booking_user_email_alert' => [
						'name'    => 'confirmed_booking_user_email_alert',
						'label'   => esc_html__( 'Confirmed Booking User Email Alert', 'cbxrbooking' ),
						'desc'    => esc_html__( 'User gets email for booking request confirmed', 'cbxrbooking' ),
						'type'    => 'title',
						'default' => ''
					],
					'confirmed_subject'                  => [
						'name'           => 'confirmed_subject',
						'label'          => esc_html__( 'Booking Request Confirmed Email Subject', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email subject user will receive when admin confirmed booking request.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Request Confirmed Notification', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'confirmed_heading'                  => [
						'name'           => 'confirmed_heading',
						'label'          => esc_html__( 'Booking Request Confirmed Email Heading', 'cbxrbooking' ),
						'desc'           => __( 'Email heading user will receive when admin confirmed booking request.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Request Confirmed', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
				],
				'cbxrbooking_cancel_booking'   => [
					'booking_cancel'      => [
						'name'    => 'booking_cancel',
						'label'   => esc_html__( 'Booking Cancel', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Enable/Disable capability if guest can cancel booking',
							'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => '',

					],
					'cancel_status'       => [
						'name'    => 'cancel_status',
						'label'   => esc_html__( 'Default Cancel Status', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Default status when user cancel a booking.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => 'canceled',
						'options' => [
							'canceled'       => esc_html__( 'Cancelled', 'cbxrbooking' ),
							'cancel-request' => esc_html__( 'Cancel Request', 'cbxrbooking' )
						]
					],
					'early_cancel'        => [
						'name'    => 'early_cancel',
						'label'   => esc_html__( 'Early Cancel Booking', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Select how early customers can cancel their booking.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => '',
						'options' => $early_cancels
					],
					'cancel_admin_alert'  => [
						'name'    => 'cancel_admin_alert',
						'label'   => esc_html__( 'Cancel Request Admin Alert', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Admin get email when user request to cancel booking', 'cbxrbooking' ),
						'type'    => 'title',
						'default' => ''
					],
					'cancel_admin_status' => [
						'name'    => 'cancel_admin_status',
						'label'   => __( 'Email Alert Status', 'cbxrbooking' ),
						'desc'    => __( 'Enable/disable email alert. Admin will receive email alert when user request to cancel a booking.',
							'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => ''
					],

					'format' => [
						'name'    => 'format',
						'label'   => esc_html__( 'E-mail Format', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Select the format of the E-mail.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => 'html',
						'options' => [
							'html'  => esc_html__( 'HTML', 'cbxrbooking' ),
							'plain' => esc_html__( 'Plain', 'cbxrbooking' )
						]
					],

					'to'                => [
						'name'           => 'to',
						'label'          => esc_html__( 'To Email', 'cbxrbooking' ),
						'desc'           => esc_html__( 'To Email Address.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => get_bloginfo( 'admin_email' ),
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'reply_to'          => [
						'name'           => 'reply_to',
						'label'          => esc_html__( 'Reply To', 'cbxrbooking' ),
						'desc'           => wp_kses( __( 'Reply To Email Address. Syntax available - <code>{user_email}</code>',
							'cbxrbooking' ), [ 'code' => [] ] ),
						'type'           => 'text',
						'default'        => '{user_email}',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'subject'           => [
						'name'           => 'subject',
						'label'          => esc_html__( 'Subject', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email Subject.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Cancel Request Notification', 'cbxrbooking' ),
						'desc_tip'       => true,
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'heading'           => [
						'name'           => 'heading',
						'label'          => esc_html__( 'Heading', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email Template Heading.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Cancel Request', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'cc'                => [
						'name'           => 'cc',
						'label'          => esc_html__( 'CC', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email CC, for multiple use comma.', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => '',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'bcc'               => [
						'name'           => 'bcc',
						'label'          => esc_html__( 'BCC', 'cbxrbooking' ),
						'desc'           => esc_html__( 'Email BCC, for multiple use comma', 'cbxrbooking' ),
						'type'           => 'text',
						'default'        => '',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'cancel_user_alert' => [
						'name'    => 'cancel_user_alert',
						'label'   => esc_html__( 'Cancel Approve User Alert', 'cbxrbooking' ),
						'desc'    => esc_html__( 'User request for cancel booking, user will get either request processing email alert or request confirm alert when admin approve or default request status is cancelled.',
							'cbxrbooking' ),
						'type'    => 'title',
						'default' => ''
					],
					'format_user_mail'  => [
						'name'    => 'format_user_mail',
						'label'   => esc_html__( 'E-mail Format', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Select the format of the E-mail.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => 'html',
						'options' => [
							'html'  => esc_html__( 'HTML', 'cbxrbooking' ),
							'plain' => esc_html__( 'Plain', 'cbxrbooking' )
						]
					],

					'to_user_mail'                 => [
						'name'           => 'to_user_mail',
						'label'          => __( 'To Email', 'cbxrbooking' ),
						'desc'           => __( 'To Email Address.  Syntax available - <code>{user_email}</code>',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => '{user_email}',
						'label_selector' => false,
						'value_selector' => true,
						'show_type'      => [ 'email' ],
					],
					'cancel_user_alert_progress'   => [
						'name'    => 'cancel_user_alert_progress',
						'label'   => esc_html__( 'Cancel Request User Alert-Received', 'cbxrbooking' ),
						'desc'    => '',
						'type'    => 'subtitle',
						'default' => ''
					],
					'cancel_user_status_progress'  => [
						'name'    => 'cancel_user_status_progress',
						'label'   => __( 'Request Received Email Alert Status', 'cbxrbooking' ),
						'desc'    => __( 'Enable/disable email alert. If not set auto cancel booking as per user request then user will get email alert saying booking cancel request is in progress.',
							'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => ''
					],
					'cancel_request_subject'       => [
						'name'           => 'cancel_request_subject',
						'label'          => __( 'Booking Cancel Request Email Subject', 'cbxrbooking' ),
						'desc'           => __( 'Email subject user will receive when cancel request is in processing',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Cancel Request Received Notification',
							'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'cancel_request_heading'       => [
						'name'           => 'cancel_request_heading',
						'label'          => __( 'Booking Cancel Request Email Heading', 'cbxrbooking' ),
						'desc'           => __( 'Email heading user will receive when cancel request is in processing.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Cancel Request Received', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => [],
					],
					'cancel_user_alert_cancelled'  => [
						'name'    => 'cancel_user_alert_cancelled',
						'label'   => esc_html__( 'Cancel Request User Alert-Approved', 'cbxrbooking' ),
						'desc'    => '',
						'type'    => 'subtitle',
						'default' => ''
					],
					'cancel_user_status_cancelled' => [
						'name'    => 'cancel_user_status_cancelled',
						'label'   => __( 'Request Cancelled Email Alert Status', 'cbxrbooking' ),
						'desc'    => __( 'Enable/disable email alert. If set auto cancel on request or admin accept cancel request  then user will get email alert saying booking is cancelled.',
							'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => ''
					],
					'canceled_subject'             => [
						'name'           => 'canceled_subject',
						'label'          => __( 'Booking Request Canceled Email Subject', 'cbxrbooking' ),
						'desc'           => __( 'Email subject user will receive when admin canceled booking request.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Request Canceled Notification', 'cbxrbooking' ),
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => []
					],
					'canceled_heading'             => [
						'name'           => 'canceled_heading',
						'label'          => __( 'Booking Request Canceled Email Heading', 'cbxrbooking' ),
						'desc'           => __( 'Email heading user will receive when admin canceled booking request.',
							'cbxrbooking' ),
						'type'           => 'text',
						'default'        => esc_html__( 'Booking Request Canceled', 'cbxrbooking' ),
						'desc_tip'       => true,
						'label_selector' => true,
						'value_selector' => true,
						'show_type'      => []
					]
				],
				'cbxrbooking_misc'             => [

					'showform_successful'          => [
						'name'    => 'showform_successful',
						'label'   => __( 'Form show (after successful submission)', 'cbxrbooking' ),
						'desc'    => __( 'Show form after successful submission.', 'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => $showform_successful
					],
					'show_credit'                  => [
						'name'    => 'show_credit',
						'label'   => __( 'Show Credit Under Form', 'cbxrbooking' ),
						'desc'    => __( 'Show Credit Under Form.', 'cbxrbooking' ),
						'type'    => 'radio',
						'options' => [
							'yes' => 'Yes',
							'no'  => 'No'
						],
						'default' => $show_credit
					],
					'enable_form_submission_limit' => [
						'name'    => 'enable_form_submission_limit',
						'label'   => esc_html__( 'Enable/Disable', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Enable/Disable Booking Limit.', 'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => 'off'
					],
					'form_submission_limit_val'    => [
						'name'    => 'form_submission_limit_val',
						'label'   => esc_html__( 'Max Public Booking Count Limit', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Booking form will not display if booking count is crossed.', 'cbxrbooking' ),
						'type'    => 'number',
						'default' => 0
					],
					'limit_error_message'          => [
						'name'    => 'limit_error_message',
						'label'   => esc_html__( 'Limit Crossed Message', 'cbxrbooking' ),
						'desc'    => esc_html__( 'General message after form successful submission.',
							'cbxrbooking' ),
						'type'    => 'wysiwyg',
						'default' => 'Sorry! Booking limit has crossed. We are not accepting any more request. Thank you.'
					]
				]
			];


		$settings_meta_fields = []; //final setting array that will be passed to different filters

		$meta_sections = $this->cbxrbookingform_meta_settings_sections();

		foreach ( $meta_sections as $meta_section ) {
			if ( ! isset( $settings_builtin_fields[ $meta_section['id'] ] ) ) {
				$settings_builtin_fields[ $meta_section['id'] ] = [];
			}
		}


		foreach ( $meta_sections as $meta_section ) {
			$settings_builtin_fields_meta_section        = $settings_builtin_fields[ $meta_section['id'] ];
			$settings_meta_fields[ $meta_section['id'] ] = apply_filters( 'cbxrbooking_meta_' . $meta_section['id'] . '_fields',
				$settings_builtin_fields_meta_section );
		}


		return $settings_meta_fields;
	}//end method cbxrbookingform_meta_settings_fields_by_id

	/**
	 * Save email/notification setting
	 *
	 * @return void
	 */
	public function save_email_setting() {
		if ( isset( $_REQUEST['cbxrbooking_email_edit'] ) ) {
			$email_id = isset( $_POST['email_id'] ) ? sanitize_text_field( wp_unslash( $_POST['email_id'] ) ) : '';
			$nonce    = isset( $_POST['_wpnonce'] ) ? sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Recommended
			if ( $email_id != '' ) {
				if ( ! wp_verify_nonce( $nonce, 'cbxrbooking_email_edit_' . $email_id ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
					die( esc_html__( 'Security check failed!', 'cbxrbooking' ) );
				} else {
					// Do stuff here.
					$admin_url    = admin_url( 'admin.php?page=cbxrbooking-emails' );
					$redirect_url = add_query_arg( [ 'edit' => $email_id ], $admin_url );

					$mail_helper = cbxrbooking_mailer();
					$emails      = $mail_helper->emails;
					$email       = $emails[ $email_id ];
					$form_fields = $email->form_fields;
					$settings    = $email->settings;

					foreach ( $form_fields as $field_key => $form_field ) {
						if ( isset( $_POST[ $field_key ] ) ) {
							$type = $form_field['type'];
							if ( $type == 'checkbox' ) {
								$settings[ $field_key ] = sanitize_text_field( wp_unslash( $_POST[ $field_key ] ) );
							} elseif ( $type == 'textarea' ) {
								$settings[ $field_key ] = sanitize_textarea_field( wp_unslash( $_POST[ $field_key ] ) );
							} else {
								$settings[ $field_key ] = sanitize_text_field( wp_unslash( $_POST[ $field_key ] ) );
							}
						} else {
							$settings[ $field_key ] = $form_field['default'];
						}
					}

					$email_options = get_option( 'cbxrbooking_emails', [] );

					$email_options[ $email_id ] = $settings;
					update_option( 'cbxrbooking_emails', $email_options );

					wp_safe_redirect( $redirect_url );
					exit;
				}
			} else {
				die( esc_html__( 'Sorry, invalid email id', 'cbxrbooking' ) );
			}
		}
	}//end method save_email_setting

	/**
	 * Show notice about pro addon deactivation
	 *
	 * @return void
	 * @since 2.0.0
	 */
	public function check_pro_addon() {
        //pro addon
		cbxrbooking_check_version_and_deactivate_plugin( 'cbxrbookingpro/cbxrbookingpro.php', '2.0.0', 'cbxrbooking_proaddon_deactivated' );

        //frontend addon
		cbxrbooking_version_and_deactivate_plugin('cbxrbookingfrontendlogaddon/cbxrbookingfrontendlogaddon.php', 'cbxrbooking_frontendaddon_deactivated');
	}//end method check_pro_addon
}//end class CBXRBookingAdmin