<?php

namespace CBX\RBooking;

use CBX\RBooking\Widgets\Block\RBookingBlockWidget;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}


/**
 * This class handles the gutenberg blocks
 *
 * Class CBXRBookingBlock
 */
class CBXRBookingBlock {
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;
	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->plugin_name = CBXRBOOKING_PLUGIN_NAME;
		$this->version     = CBXRBOOKING_PLUGIN_VERSION;

		global $wp_version;

		if ( version_compare( $wp_version, '5.8' ) >= 0 ) {
			add_filter( 'block_categories_all', [ $this, 'gutenberg_block_categories' ], 10, 2 );
		} else {
			add_filter( 'block_categories', [ $this, 'gutenberg_block_categories' ], 10, 2 );
		}

		add_action( 'init', [ $this, 'gutenberg_blocks' ] );
	}//end constructor


	/**
	 * Register New Gutenberg block Category if need
	 *
	 * @param $categories
	 * @param $post
	 *
	 * @return mixed
	 */
	public function gutenberg_block_categories( $categories, $post ) {
		$found = false;
		foreach ( $categories as $category ) {
			if ( $category['slug'] == 'cbxrbooking' ) {
				$found = true;
				break;
			}
		}

		if ( ! $found ) {
			return array_merge(
				$categories,
				[
					[
						'slug'  => 'cbxrbooking',
						'title' => esc_html__( 'Comfort Restaurant Booking Blocks', 'cbxrbooking' ),
					],
				]
			);
		}

		return $categories;
	}//end gutenberg_block_categories

	/**
	 * Init all gutenberg blocks
	 */
	public function gutenberg_blocks() {
		if ( ! function_exists( 'register_block_type' ) ) {
			// Gutenberg is not active.
			return;
		}

		new RBookingBlockWidget();
	}//end gutenberg_blocks
}//end class CBXRBookingBlock