<?php

namespace CBX\RBooking;

use CBX\RBooking\Api\CBXRoute;
use CBX\RBooking\Helpers\CBXRBookingHelper;

class CBXRBookingHooks {

	private $public;
	private $admin;
	private $api_routes;
	private $misc;

	public function __construct() {
		$this->public     = new CBXRBookingPublic();
		$this->admin      = new CBXRBookingAdmin();
		$this->api_routes = new CBXRoute();
		$this->misc       = new CBXRBookingMisc();

		$this->define_common_hooks();
		$this->define_admin_hooks();
		$this->define_public_hooks();


		new CBXRBookingBlock();
	}//end constructor

	/**
	 * Define common
	 *
	 * @return void
	 */
	private function define_common_hooks() {
		$route  = $this->api_routes;
		$misc   = $this->misc;
		$helper = new CBXRBookingHelper();

		add_action( 'init', [ $helper, 'load_orm' ] );
		add_action( 'plugins_loaded', [ $misc, 'load_plugin_textdomain' ] );
		add_action( 'rest_api_init', [ $route, 'init' ] );
		add_action( 'init', [ $misc, 'load_emails' ] );
		add_filter( 'script_loader_tag', [ $misc, 'add_module_to_script' ], 10, 3 );
	}//end method define_common_hooks

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$public = new CBXRBookingPublic();

		add_action( 'template_redirect', [ $public, 'frontend_cancel_request' ] );
		add_action( 'template_redirect', [ $public, 'frontend_guest_activation' ] );

		add_action( 'init', [ $public, 'init_shortcodes' ] );

		add_action( 'wp_ajax_rbooking_frontend_entrysubmit_action', [ $public, 'frontend_entrysubmit' ] );
		add_action( 'wp_ajax_nopriv_rbooking_frontend_entrysubmit_action', [ $public, 'frontend_entrysubmit' ] );

		//widget
		add_action( 'widgets_init', [ $public, 'register_widget' ] );

		//this is disabled from here but added from shortcode method as this is not needed everywhere in frontend
		add_action( 'wp_enqueue_scripts', [ $public, 'enqueue_styles' ] );
		add_action( 'wp_enqueue_scripts', [ $public, 'enqueue_scripts' ] );

		//elementor
		//Elementor Widget
		add_action( 'elementor/widgets/widgets_registered', [ $public, 'init_elementor_widgets' ] );
		add_action( 'elementor/elements/categories_registered', [ $public, 'add_elementor_widget_categories' ] );
		add_action( 'elementor/editor/before_enqueue_scripts', [ $public, 'elementor_icon_loader' ], 99999 );
	}//end method define_public_hooks

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$admin = new CBXRBookingAdmin();
		$misc  = new CBXRBookingMisc();


		//adding the setting action
		add_action( 'admin_init', [ $admin, 'setting_init' ], 0 );

		//add metabox for custom post type cbxrbooking
		add_action( 'init', [ $admin, 'create_cbxrbooking_post_type' ], 0 );

		//Add admin menu action hook; for adding setting menu
		add_action( 'admin_menu', [ $admin, 'admin_menus' ], 11 );

		add_action( 'restrict_manage_posts', [ $admin, 'admin_posts_filter_add_branch_dropdown' ] );
		add_action( 'parse_query', [ $admin, 'admin_posts_parse_query_modification' ] );

		// for enqueue scripts and styles
		add_action( 'admin_enqueue_scripts', [ $admin, 'enqueue_styles' ] );
		add_action( 'admin_enqueue_scripts', [ $admin, 'enqueue_scripts' ] );

		add_action( 'admin_init', [ $admin, 'save_email_setting' ] );

		//upgrade and admin notice
		add_action( 'upgrader_process_complete', [ $admin, 'plugin_upgrader_process_complete' ], 10, 2 );
		add_action( 'admin_notices', [ $admin, 'plugin_activate_upgrade_notices' ] );
		add_filter( 'plugin_action_links_' . CBXRBOOKING_BASE_NAME, [ $admin, 'plugin_action_links' ] );
		add_filter( 'plugin_row_meta', [ $admin, 'plugin_row_meta' ], 10, 4 );

		add_action( 'after_plugin_row_cbxrbookingpro/cbxrbookingpro.php', [
			$misc,
			'message_after_plugin_row_proaddon'
		], 10, 2 );


		add_action( 'activated_plugin', [ $admin, 'check_pro_addon' ] );
		add_action( 'init', [ $admin, 'check_pro_addon' ] );

		add_filter( 'robots_txt', [ $misc, 'custom_robots_txt' ] );
	}//end method define_admin_hooks
}//end class CBXRBookingHooks