<?php
namespace CBX\RBooking;

/**
 * Class CBXRBookingMisc
 *
 * Miscellaneous features
 */
class  CBXRBookingMisc {
	public function __construct() {

	}//end constructor

	/**
	 * load emails
	 */
	public function load_emails() {
		cbxrbooking_mailer();
	} //end method load_emails

	/**
	 * Load the plugin text domain for translation.
	 *
	 * @since    1.0.0
	 */
	public function load_plugin_textdomain() {
		load_plugin_textdomain( 'cbxrbooking', false, CBXRBOOKING_ROOT_PATH . 'languages/' );
	}//end method load_plugin_textdomain

	/**
	 * Add module attribute to script loader
	 *
	 * @param $tag
	 * @param $handle
	 * @param $src
	 *
	 * @return mixed|string
	 */
	public function add_module_to_script( $tag, $handle, $src ) {
		$jsHandles = [
			'cbxrbooking_form_vue_dev',
			'cbxrbooking_form_vue_main',
			'cbxrbooking_log_vue_dev',
			'cbxrbooking_log_vue_main',
			'cbxrbooking_dashboard_vue_dev',
			'cbxrbooking_dashboard_vue_main',
			'cbxrbooking_tools_vue_dev',
			'cbxrbooking_tools_vue_main',
			'cbxrbooking_branch_vue_dev',
			'cbxrbooking_branch_vue_main',
		];

		if ( in_array( $handle, $jsHandles ) ) {
			$tag = '<script type="module" id="' . $handle . '" src="' . esc_url( $src ) . '"></script>';
		}

		return $tag;
	}//end method add_module_to_script

	/**
	 * Tell bots not to index some created directories.
	 *
	 * We try to detect the default "User-agent: *" added by WordPress and add our rules to that group, because
	 * it's possible that some bots will only interpret the first group of rules if there are multiple groups with
	 * the same user agent.
	 *
	 * @param string $output The contents that WordPress will output in a robots.txt file.
	 *
	 * @return string
	 */
	private function custom_robots_txt( $output ) {
		$site_url = wp_parse_url( site_url() );
		$path     = ( ! empty( $site_url['path'] ) ) ? $site_url['path'] : '';

		$lines       = preg_split( '/\r\n|\r|\n/', $output );
		$agent_index = array_search( 'User-agent: *', $lines, true );

		if ( false !== $agent_index ) {
			$above = array_slice( $lines, 0, $agent_index + 1 );
			$below = array_slice( $lines, $agent_index + 1 );
		} else {
			$above   = $lines;
			$below   = [];
			$above[] = '';
			$above[] = 'User-agent: *';
		}

		$above[] = "Disallow: $path/wp-content/uploads/cbxrbooking/";

		$lines = array_merge( $above, $below );

		return implode( PHP_EOL, $lines );
	}//end method custom_robots_txt

	/**
	 * Show plugin update
	 *
	 * @param $plugin_file
	 * @param $plugin_data
	 *
	 * @return void
	 */
	public function message_after_plugin_row_proaddon( $plugin_file, $plugin_data ) {
		if ( $plugin_file !== 'cbxrbookingpro/cbxrbookingpro.php' ) {
			return;
		}

		if ( defined( 'CBXRBOOKINGPRO_PLUGIN_NAME' ) ) {
			return;
		}

		//$pro_addon_version = CBXRBookingHelpers::get_any_plugin_version('cbxrbookingpro/cbxrbookingpro.php');
		$pro_addon_version = isset( $plugin_data['Version'] ) ? $plugin_data['Version'] : '';
		$required_version = '2.0.0';

		if ( $pro_addon_version != '' && version_compare( $pro_addon_version, $required_version, '<' ) ) {
			// Custom message to display

			//$plugin_setting_url = admin_url( 'admin.php?page=cbxwpbookmark_settings#cbxwpbookmark_licences' );
			$manual_update_url = 'https://codeboxr.com/manual-update-pro-addon/';

			/* translators:translators: %s: plugin setting url for licence */
			$custom_message = wp_kses( sprintf( __( '<strong>Note:</strong> Comfort Restaurant Booking Pro Addon is custom plugin. This plugin can not be auto update from dashboard/plugin manager. For manual update please check <a target="_blank" href="%1$s">documentation</a>. <strong style="color: red;">It seems this plugin\'s current version is older than %2$s . To get the latest pro addon features, this plugin needs to upgrade to %2$s or later.</strong>',
				'cbxrbooking' ), esc_url( $manual_update_url, $required_version ) ), [
				'strong' => [ 'style' => [] ],
				'a'      => [
					'href'   => [],
					'target' => []
				]
			] );

			// Output a row with custom content
			echo '<tr class="plugin-update-tr">
            <td colspan="3" class="plugin-update colspanchange">
                <div class="notice notice-warning inline">
                    ' . wp_kses_post( $custom_message ) . '
                </div>
            </td>
          </tr>';
		}
	}//end method message_after_plugin_row_proaddon
}//end class CBXRBookingMisc
