<?php

namespace CBX\RBooking\Helpers;

use CBX\RBooking\CBXRBookingAdmin;
use CBX\RBooking\Models\Post;
use CBX\RBooking\Models\RBookingLog;
use CBX\RBooking\Models\RBranch;
use DateTime;
use DateTimeZone;
use Illuminate\Database\Capsule\Manager;
use CBX\RBooking\MigrationManage;

if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * The phelper functionality of the plugin.
 *
 * @link       http://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXRBooking
 * @subpackage CBXRBooking/includes
 */

/**
 * Class CBXRBookingHelper
 *
 */
class CBXRBookingHelper {

	/**
	 * On plugin activate
	 */
	public static function activate() {
		/*//check if can activate plugin
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		$plugin = isset( $_REQUEST['plugin'] ) ? sanitize_text_field(wp_unslash($_REQUEST['plugin'])) : '';
		check_admin_referer( "activate-plugin_{$plugin}" );*/

		//set the current version
		update_option( 'cbxrbooking_version', CBXRBOOKING_PLUGIN_VERSION );

		self::migration_and_defaults();

		set_transient( 'cbxrbooking_activated_notice', 1 );
	}//end method activate

	/**
	 * On plugin deactivate
	 */
	public static function deactivate() {

	}//end method deactivate

	/**
	 * Create default role and capability on plugin activation and rest
	 *
	 * @since 1.0.0
	 */
	private function defaultRoleCapability() {
		//cbxrbooking capabilities list
		$caps = cbxrbooking_all_caps();

		//add the caps to the administrator role
		$role = get_role( 'administrator' );

		foreach ( $caps as $cap ) {
			//add cap to the role
			if ( ! $role->has_cap( $cap ) ) {
				// add a custom capability
				$role->add_cap( $cap, true );

			}

			//update the same cap for the current user who is installing or updating if logged in
			$this->update_user_capability( $cap );
		}
	}//end method defaultRoleCapability

	/**
	 * Add any capability to the current user
	 *
	 * @param $capability_to_add
	 *
	 * @return void
	 */
	private function update_user_capability( $capability_to_add ) {
		// Check if a user is logged in.
		if ( is_user_logged_in() ) {
			// Get the current user object.
			$user = wp_get_current_user();

			// Check if the user already has the capability.
			if ( ! $user->has_cap( $capability_to_add ) ) {
				// Add the capability.
				$user->add_cap( $capability_to_add );

				// Optional: Log the capability addition (for debugging or auditing).
				//error_log( 'Added capability "' . $capability_to_add . '" to user: ' . $user->user_login );

				// Optional: Force a refresh of the user's capabilities (sometimes needed).
				wp_cache_delete( $user->ID, 'users' );
				wp_cache_delete( 'user_meta', $user->ID );

			} else {
				// Optional: Log that the user already has the capability.
				//error_log( 'User: ' . $user->user_login . ' already has capability: ' . $capability_to_add );
			}
		} else {
			// Optional: Handle the case where no user is logged in.
			//error_log( 'No user is logged in.' );
		}
	}//end method update_user_capability

	/**
	 * On plugin active or reset data set default data for this plugin
	 *
	 * @since 1.0.0
	 */
	public function default_data_set() {

		// add role and custom capability
		$this->defaultRoleCapability();
	}//end method default_data_set

	/**
	 * On plugin activate run migration
	 */
	public static function migration_and_defaults() {

		MigrationManage::run();

		//set default data
		( new self() )->default_data_set();
	}//end method migration_and_defaults

	/**
	 * Load ORM
	 *
	 * @since  1.0.0
	 */
	public static function load_orm() {

		/**
		 * Init DB in ORM
		 */
		global $wpdb;

		$capsule = new Manager();

		$connection_params = [
			'driver'   => 'mysql',
			'host'     => DB_HOST,
			'database' => DB_NAME,
			'username' => DB_USER,
			'password' => DB_PASSWORD,
			'prefix'   => $wpdb->prefix,
		];

		if ( $wpdb->has_cap( 'collation' ) ) {
			if ( DB_CHARSET != '' ) {
				$connection_params['charset'] = DB_CHARSET;
			}

			if ( DB_COLLATE != '' ) {
				$connection_params['collation'] = DB_COLLATE;
			}
		}

		$capsule->addConnection( apply_filters( 'cbxrbooking_database_connection_params', $connection_params ) );

		$capsule->setAsGlobal();
		$capsule->bootEloquent();
	} //end method load_orm

	/**
	 * Get all  core tables list
	 */
	public static function getAllDBTablesList() {
		global $wpdb;

		$table_names                               = [];
		$table_names['cbxrbooking_log_manager']    = $wpdb->prefix . 'cbxrbooking_log_manager';
		$table_names['cbxrbooking_branch_manager'] = $wpdb->prefix . 'cbxrbooking_branch_manager';


		return apply_filters( 'cbxrbooking_table_list', $table_names );
	}//end getAllDBTablesList

	/**
	 * List all global option name with prefix cbxpoll_
	 */
	public static function getAllOptionNames() {
		global $wpdb;

		$prefix       = 'cbxrbooking_';
		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		$option_names = $wpdb->get_results( "SELECT * FROM {$wpdb->options} WHERE option_name LIKE '{$prefix}%'",
			ARRAY_A );

		return apply_filters( 'cbxrbooking_option_names', $option_names );
	}

	/**
	 * return all date format
	 *
	 * @return mixed|void
	 */
	public static function getAllDateFormat() {
		$date_format_arr = [
			'0' => esc_html__( 'Y-m-d', 'cbxrbooking' ),
			'1' => esc_html__( 'Y-M-D', 'cbxrbooking' ),
			'2' => esc_html__( 'd-m-y', 'cbxrbooking' ),
		];

		return apply_filters( 'cbxrbooking_all_date_format', $date_format_arr );
	}//end getAllDateFormat

	/**
	 * Return single date format
	 *
	 * @param  int  $format_key
	 *
	 * @return mixed
	 */
	public static function getSingleDateFormat( $format_key = 0 ) {
		$date_format_arr = self::getAllDateFormat();
		if ( is_array( $date_format_arr ) && isset( $date_format_arr[ $format_key ] ) ) {
			$single_date_format = $date_format_arr[ $format_key ];
		} else {
			$single_date_format = $date_format_arr[0];
		}

		return $single_date_format;
	}

	/**
	 * global stored date format key from setting
	 * @return int
	 */
	public static function storedGlobalDateFormatKey() {
		$cbxrb_global_settings = get_option( 'cbxrbooking_global' );

		$global_date_format_key = 0;
		if ( $cbxrb_global_settings !== false && isset( $cbxrb_global_settings['global_date_format'] ) ) {
			$global_date_format_key = intval( $cbxrb_global_settings['global_date_format'] );
		}

		return $global_date_format_key;
	}

	/**
	 * form stored date format key from setting
	 * @return int
	 */
	public static function storedFormDateFormatKey( $post_id = 0 ) {

		$form_date_format_key = 0;

		$post_id = intval( $post_id );
		if ( $post_id > 0 ) {
			$cbxrbooking_meta = get_post_meta( $post_id, '_cbxrbookingmeta', true );
			if ( is_array( $cbxrbooking_meta ) && sizeof( $cbxrbooking_meta ) > 0 ) {
				if ( isset( $cbxrbooking_meta['settings']['cbxrbooking_booking_schedule']['date_format'] ) ) {
					$form_date_format_key = intval( $cbxrbooking_meta['settings']['cbxrbooking_booking_schedule']['date_format'] );
				}
			}
		}

		return $form_date_format_key;
	}

	/**
	 * view date format
	 *
	 * @param  string  $timestamp
	 * @param  int  $date_format_key
	 *
	 * @return false|string
	 */
	public static function viewDateFormat( $timestamp = '', $date_format_key = 0 ) {

		$date_format = self::getSingleDateFormat( $date_format_key );

		return gmdate( $date_format, strtotime( $timestamp ) );
	}


	/**
	 * Returns all booking status
	 *
	 * @return array
	 */
	public static function getAllBookingStatus() {

		$status = [
			'unverified'     => esc_html__( 'Unverified', 'cbxrbooking' ),
			'pending'        => esc_html__( 'Pending', 'cbxrbooking' ),
			'confirmed'      => esc_html__( 'Confirmed', 'cbxrbooking' ),
			'canceled'       => esc_html__( 'Cancelled', 'cbxrbooking' ),
			'cancel-request' => esc_html__( 'Cancel Requested', 'cbxrbooking' ),
			'waiting'        => esc_html__( 'Waiting', 'cbxrbooking' ),
			'seated'         => esc_html__( 'Seated', 'cbxrbooking' ),
			'departed'       => esc_html__( 'Departed', 'cbxrbooking' ),
			'closed'         => esc_html__( 'Closed', 'cbxrbooking' ),
			'archived'       => esc_html__( 'Archived', 'cbxrbooking' ),
			'trashed'        => esc_html__( 'Trashed', 'cbxrbooking' ),
		];

		return apply_filters( 'cbxrbooking_status', $status );
	}//end getAllBookingStatus

	/**
	 * Returns booking status
	 *
	 * @return array
	 */
	public static function getBookingStatus( $key ) {
		$all_status = self::getAllBookingStatus();

		return isset( $all_status[ $key ] ) ? $all_status[ $key ] : esc_html__( 'Unknown Status', 'cbxrbooking' );
	}//end getBookingStatus

	/**
	 * Get IP Address
	 *
	 * @return string|void
	 * return ip address
	 */
	public static function get_ipaddress() {

		if ( empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {

			$ip_address = isset($_SERVER['REMOTE_ADDR']) ? sanitize_text_field( wp_unslash($_SERVER['REMOTE_ADDR'])): '';
		} else {

			$ip_address = sanitize_text_field( wp_unslash($_SERVER['HTTP_X_FORWARDED_FOR']));
		}

		if ( strpos( $ip_address, ',' ) !== false ) {

			$ip_address = explode( ',', $ip_address );
			$ip_address = $ip_address[0];
		}

		return esc_attr( $ip_address );
	}

	public static function getCountry( $key ) {
		$countries = self::getAllCountries();
		if ( $key == '' || $key == 'none' ) {
			return '';
		}

		if ( isset( $countries[ $key ] ) ) {
			return $countries[ $key ];
		} else {
			'';
		}
	}

	/**
	 * Get all countries
	 *
	 * @return array
	 */
	public static function getAllCountries() {
		$countries = [
			'AF' => 'Afghanistan',
			'AL' => 'Albania',
			'DZ' => 'Algeria',
			'DS' => 'American Samoa',
			'AD' => 'Andorra',
			'AO' => 'Angola',
			'AI' => 'Anguilla',
			'AQ' => 'Antarctica',
			'AG' => 'Antigua and Barbuda',
			'AR' => 'Argentina',
			'AM' => 'Armenia',
			'AW' => 'Aruba',
			'AU' => 'Australia',
			'AT' => 'Austria',
			'AZ' => 'Azerbaijan',
			'BS' => 'Bahamas',
			'BH' => 'Bahrain',
			'BD' => 'Bangladesh',
			'BB' => 'Barbados',
			'BY' => 'Belarus',
			'BE' => 'Belgium',
			'BZ' => 'Belize',
			'BJ' => 'Benin',
			'BM' => 'Bermuda',
			'BT' => 'Bhutan',
			'BO' => 'Bolivia',
			'BA' => 'Bosnia and Herzegovina',
			'BW' => 'Botswana',
			'BV' => 'Bouvet Island',
			'BR' => 'Brazil',
			'IO' => 'British Indian Ocean Territory',
			'BN' => 'Brunei Darussalam',
			'BG' => 'Bulgaria',
			'BF' => 'Burkina Faso',
			'BI' => 'Burundi',
			'KH' => 'Cambodia',
			'CM' => 'Cameroon',
			'CA' => 'Canada',
			'CV' => 'Cape Verde',
			'KY' => 'Cayman Islands',
			'CF' => 'Central African Republic',
			'TD' => 'Chad',
			'CL' => 'Chile',
			'CN' => 'China',
			'CX' => 'Christmas Island',
			'CC' => 'Cocos (Keeling) Islands',
			'CO' => 'Colombia',
			'KM' => 'Comoros',
			'CG' => 'Congo',
			'CK' => 'Cook Islands',
			'CR' => 'Costa Rica',
			'HR' => 'Croatia (Hrvatska)',
			'CU' => 'Cuba',
			'CY' => 'Cyprus',
			'CZ' => 'Czech Republic',
			'DK' => 'Denmark',
			'DJ' => 'Djibouti',
			'DM' => 'Dominica',
			'DO' => 'Dominican Republic',
			'TP' => 'East Timor',
			'EC' => 'Ecuador',
			'EG' => 'Egypt',
			'SV' => 'El Salvador',
			'GQ' => 'Equatorial Guinea',
			'ER' => 'Eritrea',
			'EE' => 'Estonia',
			'ET' => 'Ethiopia',
			'FK' => 'Falkland Islands (Malvinas)',
			'FO' => 'Faroe Islands',
			'FJ' => 'Fiji',
			'FI' => 'Finland',
			'FR' => 'France',
			'FX' => 'France, Metropolitan',
			'GF' => 'French Guiana',
			'PF' => 'French Polynesia',
			'TF' => 'French Southern Territories',
			'GA' => 'Gabon',
			'GM' => 'Gambia',
			'GE' => 'Georgia',
			'DE' => 'Germany',
			'GH' => 'Ghana',
			'GI' => 'Gibraltar',
			'GK' => 'Guernsey',
			'GR' => 'Greece',
			'GL' => 'Greenland',
			'GD' => 'Grenada',
			'GP' => 'Guadeloupe',
			'GU' => 'Guam',
			'GT' => 'Guatemala',
			'GN' => 'Guinea',
			'GW' => 'Guinea-Bissau',
			'GY' => 'Guyana',
			'HT' => 'Haiti',
			'HM' => 'Heard and Mc Donald Islands',
			'HN' => 'Honduras',
			'HK' => 'Hong Kong',
			'HU' => 'Hungary',
			'IS' => 'Iceland',
			'IN' => 'India',
			'IM' => 'Isle of Man',
			'ID' => 'Indonesia',
			'IR' => 'Iran (Islamic Republic of)',
			'IQ' => 'Iraq',
			'IE' => 'Ireland',
			'IL' => 'Israel',
			'IT' => 'Italy',
			'CI' => 'Ivory Coast',
			'JE' => 'Jersey',
			'JM' => 'Jamaica',
			'JP' => 'Japan',
			'JO' => 'Jordan',
			'KZ' => 'Kazakhstan',
			'KE' => 'Kenya',
			'KI' => 'Kiribati',
			'KP' => "Korea, Democratic People's Republic of",
			'KR' => 'Korea, Republic of',
			'XK' => 'Kosovo',
			'KW' => 'Kuwait',
			'KG' => 'Kyrgyzstan',
			'LA' => "Lao People's Democratic Republic",
			'LV' => 'Latvia',
			'LB' => 'Lebanon',
			'LS' => 'Lesotho',
			'LR' => 'Liberia',
			'LY' => 'Libyan Arab Jamahiriya',
			'LI' => 'Liechtenstein',
			'LT' => 'Lithuania',
			'LU' => 'Luxembourg',
			'MO' => 'Macau',
			'MK' => 'Macedonia',
			'MG' => 'Madagascar',
			'MW' => 'Malawi',
			'MY' => 'Malaysia',
			'MV' => 'Maldives',
			'ML' => 'Mali',
			'MT' => 'Malta',
			'MH' => 'Marshall Islands',
			'MQ' => 'Martinique',
			'MR' => 'Mauritania',
			'MU' => 'Mauritius',
			'TY' => 'Mayotte',
			'MX' => 'Mexico',
			'FM' => 'Micronesia, Federated States of',
			'MD' => 'Moldova, Republic of',
			'MC' => 'Monaco',
			'MN' => 'Mongolia',
			'ME' => 'Montenegro',
			'MS' => 'Montserrat',
			'MA' => 'Morocco',
			'MZ' => 'Mozambique',
			'MM' => 'Myanmar',
			'NA' => 'Namibia',
			'NR' => 'Nauru',
			'NP' => 'Nepal',
			'NL' => 'Netherlands',
			'AN' => 'Netherlands Antilles',
			'NC' => 'New Caledonia',
			'NZ' => 'New Zealand',
			'NI' => 'Nicaragua',
			'NE' => 'Niger',
			'NG' => 'Nigeria',
			'NU' => 'Niue',
			'NF' => 'Norfolk Island',
			'MP' => 'Northern Mariana Islands',
			'NO' => 'Norway',
			'OM' => 'Oman',
			'PK' => 'Pakistan',
			'PW' => 'Palau',
			'PS' => 'Palestine',
			'PA' => 'Panama',
			'PG' => 'Papua New Guinea',
			'PY' => 'Paraguay',
			'PE' => 'Peru',
			'PH' => 'Philippines',
			'PN' => 'Pitcairn',
			'PL' => 'Poland',
			'PT' => 'Portugal',
			'PR' => 'Puerto Rico',
			'QA' => 'Qatar',
			'RE' => 'Reunion',
			'RO' => 'Romania',
			'RU' => 'Russian Federation',
			'RW' => 'Rwanda',
			'KN' => 'Saint Kitts and Nevis',
			'LC' => 'Saint Lucia',
			'VC' => 'Saint Vincent and the Grenadines',
			'WS' => 'Samoa',
			'SM' => 'San Marino',
			'ST' => 'Sao Tome and Principe',
			'SA' => 'Saudi Arabia',
			'SN' => 'Senegal',
			'RS' => 'Serbia',
			'SC' => 'Seychelles',
			'SL' => 'Sierra Leone',
			'SG' => 'Singapore',
			'SK' => 'Slovakia',
			'SI' => 'Slovenia',
			'SB' => 'Solomon Islands',
			'SO' => 'Somalia',
			'ZA' => 'South Africa',
			'GS' => 'South Georgia South Sandwich Islands',
			'ES' => 'Spain',
			'LK' => 'Sri Lanka',
			'SH' => 'St. Helena',
			'PM' => 'St. Pierre and Miquelon',
			'SD' => 'Sudan',
			'SR' => 'Suriname',
			'SJ' => 'Svalbard and Jan Mayen Islands',
			'SZ' => 'Swaziland',
			'SE' => 'Sweden',
			'CH' => 'Switzerland',
			'SY' => 'Syrian Arab Republic',
			'TW' => 'Taiwan',
			'TJ' => 'Tajikistan',
			'TZ' => 'Tanzania, United Republic of',
			'TH' => 'Thailand',
			'TG' => 'Togo',
			'TK' => 'Tokelau',
			'TO' => 'Tonga',
			'TT' => 'Trinidad and Tobago',
			'TN' => 'Tunisia',
			'TR' => 'Turkey',
			'TM' => 'Turkmenistan',
			'TC' => 'Turks and Caicos Islands',
			'TV' => 'Tuvalu',
			'UG' => 'Uganda',
			'UA' => 'Ukraine',
			'AE' => 'United Arab Emirates',
			'GB' => 'United Kingdom',
			'US' => 'United States',
			'UM' => 'United States minor outlying islands',
			'UY' => 'Uruguay',
			'UZ' => 'Uzbekistan',
			'VU' => 'Vanuatu',
			'VA' => 'Vatican City State',
			'VE' => 'Venezuela',
			'VN' => 'Vietnam',
			'VG' => 'Virgin Islands (British)',
			'VI' => 'Virgin Islands (U.S.)',
			'WF' => 'Wallis and Futuna Islands',
			'EH' => 'Western Sahara',
			'YE' => 'Yemen',
			'ZR' => 'Zaire',
			'ZM' => 'Zambia',
			'ZW' => 'Zimbabwe'
		];

		return $countries;
	}

	public static function generateBookingSecret() {
		return wp_generate_password( $length = 12, false, false );
	}//end generateBookingSecret

	/**
	 * Return all branches
	 *
	 * @return array|null|object
	 */
	public static function getAllBranches() {
		global $wpdb;
		$branch_manager_table_name = $wpdb->prefix . 'cbxrbooking_branch_manager';
		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		$all_branches = $wpdb->get_results( "SELECT id, name FROM $branch_manager_table_name", 'ARRAY_A' );

		return $all_branches;
	}//end getAllBranches

	/**
	 * Get all booking forms for any branch id
	 *
	 * @param  int  $branch_id
	 *
	 * @return array|object|null
	 */
	public static function getAllBookingForms( $branch_id = 0 ) {
		global $wpdb;
		$posts_table    = $wpdb->prefix . 'posts';
		$postmeta_table = $wpdb->prefix . 'postmeta';

		//get the booking forms
		$sql_select = "SELECT DISTINCT posts.ID as id, posts.post_title as form_name FROM $posts_table as posts ";

		$where_sql = $wpdb->prepare( 'posts.post_type = %s AND posts.post_status = %s', 'cbxrbooking', 'publish' );
		$join      = '';

		// add branch id in where clause
		if ( $branch_id > 0 ) {
			$join      .= " JOIN $postmeta_table postmeta ON posts.ID = postmeta.post_id";
			$where_sql .= ( $where_sql != '' ) ? ' AND ' : '';
			$where_sql .= $wpdb->prepare( 'postmeta.meta_key = %s AND postmeta.meta_value = %d',
				'_cbxrbookingformmeta_branch', $branch_id );
		}

		$sortingOrder  = ' ORDER BY posts.post_title desc ';
		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		$booking_forms = $wpdb->get_results( "$sql_select $join WHERE $where_sql $sortingOrder", ARRAY_A );

		return $booking_forms;
	}//end getAllBookingForms

	/**
	 * Get Data
	 *
	 * @param  string  $search
	 * @param  int  $form_id
	 * @param  string  $date_from
	 * @param  string  $date_to
	 * @param  string  $orderby
	 * @param  string  $order
	 * @param  int  $perpage
	 * @param  int  $page
	 * @param  string  $status
	 * @param  string  $date_range
	 * @param  int  $branch_id
	 *
	 * @return array|object|null
	 * @throws Exception
	 */
	public static function getLogData(
		$search = '',
		$form_id = 0,
		$date_from = '',
		$date_to = '',
		$orderby = 'id',
		$order = 'asc',
		$perpage = 20,
		$page = 1,
		$status = 'all',
		$date_range = 'all',
		$branch_id = 0
	) {

		global $wpdb;
		$current_offset = get_option( 'gmt_offset' );
		$tzstring       = get_option( 'timezone_string' );

		$check_zone_info = true;

		// Remove old Etc mappings. Fallback to gmt_offset.
		if ( false !== strpos( $tzstring, 'Etc/GMT' ) ) {
			$tzstring = '';
		}

		if ( empty( $tzstring ) ) { // Create a UTC+- zone if no timezone string exists
			$check_zone_info = false;
			if ( 0 == $current_offset ) {
				$tzstring = '+0';
			} elseif ( $current_offset < 0 ) {
				$tzstring = '' . $current_offset;
			} else {
				$tzstring = '+' . $current_offset;
			}
		}

		$date_time_zone           = new DateTimeZone( $tzstring );
		$today_datetime_requested = new DateTime( 'now', $date_time_zone );
		$today_date               = $today_datetime_requested->format( 'Y-m-d' );


		$booking_table = $wpdb->prefix . 'cbxrbooking_log_manager';

		$sql_select = "SELECT * FROM $booking_table as logs";

		$where_sql = '';

		if ( $search != '' ) {
			if ( $where_sql != '' ) {
				$where_sql .= ' AND ';
			}
			$search_like = '%' . $wpdb->esc_like( $search ) . '%';
			$where_sql .= $wpdb->prepare( " name LIKE %s OR email LIKE %s OR phone LIKE %s OR message LIKE %s OR secret LIKE %s",
				$search_like, $search_like, $search_like, $search_like, $search_like );

		}

		$join = '';

		if ( $branch_id !== 0 ) {
			$postmeta_table = $wpdb->prefix . 'postmeta'; // postmeta

			$join .= " JOIN $postmeta_table postmeta ON logs.form_id = postmeta.post_id";

			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
			$where_sql .= $wpdb->prepare( 'postmeta.meta_key = %s AND postmeta.meta_value = %d',
				'_cbxrbookingformmeta_branch', $branch_id );
		}

		if ( $form_id !== 0 ) {
			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' ) . $wpdb->prepare( 'form_id=%d', $form_id );
		}

		if ( $status !== 'all' ) {
			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' ) . $wpdb->prepare( 'status=%s', $status );
		}

		if ( $date_range !== '' ) {

			if ( $date_range == 'upcoming' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= $wpdb->prepare( 'booking_date > %s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date > %s', $today_date );
			} elseif ( $date_range == 'today' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= $wpdb->prepare( 'booking_date = %s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date = %s', $today_date );
			} elseif ( $date_range == 'previous' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= $wpdb->prepare( 'booking_date < %s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date < %s', $today_date );
			} elseif ( $date_range == 'between_dates' && $date_from != '' && $date_to != '' ) {


				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= " CAST(logs.booking_date AS DATE) ";
				$where_sql .= " CAST(concat(logs.booking_date, ' ', logs.booking_time) as datetime) ";
				$where_sql .= $wpdb->prepare( 'between %s AND %s ', $date_from, $date_to );
			}
		}

		if ( $where_sql == '' ) {
			$where_sql = '1';
		}

		$start_point = ( $page * $perpage ) - $perpage;
		$limit_sql   = 'LIMIT';
		$limit_sql   .= ' ' . $start_point . ',';
		$limit_sql   .= ' ' . $perpage;

		$sortingOrder = " ORDER BY $orderby $order ";

		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		$data = $wpdb->get_results( "$sql_select $join  WHERE  $where_sql $sortingOrder  $limit_sql", 'ARRAY_A' );

		return $data;
	}//end getLogData

	/**
	 * Get total data count
	 *
	 * @param  string  $search
	 * @param  int  $form_id
	 * @param  string  $date_from
	 * @param  string  $date_to
	 * @param  string  $orderby
	 * @param  string  $order
	 * @param  string  $status
	 * @param  string  $date_range
	 * @param  int  $branch_id
	 *
	 * @return string|null
	 * @throws Exception
	 */
	public static function getLogDataCount(
		$search = '',
		$form_id = 0,
		$date_from = '',
		$date_to = '',
		$orderby = 'id',
		$order = 'asc',
		$status = 'all',
		$date_range = 'all',
		$branch_id = 0
	) {

		global $wpdb;

		$current_offset = get_option( 'gmt_offset' );
		$tzstring       = get_option( 'timezone_string' );

		$check_zone_info = true;

		// Remove old Etc mappings. Fallback to gmt_offset.
		if ( false !== strpos( $tzstring, 'Etc/GMT' ) ) {
			$tzstring = '';
		}

		if ( empty( $tzstring ) ) { // Create a UTC+- zone if no timezone string exists
			$check_zone_info = false;
			if ( 0 == $current_offset ) {
				$tzstring = '+0';
			} elseif ( $current_offset < 0 ) {
				$tzstring = '' . $current_offset;
			} else {
				$tzstring = '+' . $current_offset;
			}
		}

		$date_time_zone           = new DateTimeZone( $tzstring );
		$today_datetime_requested = new DateTime( 'now', $date_time_zone );
		$today_date               = $today_datetime_requested->format( 'Y-m-d' );

		$booking_table = $wpdb->prefix . 'cbxrbooking_log_manager';

		$sql_select = "SELECT COUNT(*) FROM $booking_table as logs";

		$where_sql = '';

		$join = '';
		if ( $branch_id !== 0 ) {

			$postmeta_table = $wpdb->prefix . 'postmeta'; // postmeta

			$join .= " JOIN $postmeta_table postmeta ON logs.form_id = postmeta.post_id";

			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
			$where_sql .= $wpdb->prepare( 'postmeta.meta_key = %s AND postmeta.meta_value = %d',
				'_cbxrbookingformmeta_branch', $branch_id );
		}

		if ( $search != '' ) {
			if ( $where_sql != '' ) {
				$where_sql .= ' AND ';
			}
			$search_like = '%' . $wpdb->esc_like( $search ) . '%';
			$where_sql .= $wpdb->prepare( " name LIKE %s OR email LIKE %s OR phone LIKE %s OR message LIKE %s OR secret LIKE %s",
				$search_like, $search_like, $search_like, $search_like, $search_like );
		}

		if ( $form_id !== 0 ) {
			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' ) . $wpdb->prepare( 'form_id=%d', $form_id );
		}

		if ( $status != 'all' ) {
			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' ) . $wpdb->prepare( 'status=%s', $status );
		}

		if ( $date_range != 'all' ) {
			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );

			if ( $date_range === 'upcoming' ) {
				//$where_sql .= $wpdb->prepare( 'booking_date>%s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date > %s', $today_date );
			} elseif ( $date_range === 'today' ) {
				//$where_sql .= $wpdb->prepare( 'booking_date = %s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date = %s', $today_date );
			} elseif ( $date_range === 'previous' ) {
				//$where_sql .= $wpdb->prepare( 'booking_date < %s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date < %s', $today_date );
			} elseif ( $date_range === 'between_dates' && $date_from != '' && $date_to != '' ) {

				//$where_sql .= " CAST(logs.booking_date AS DATE) ";
				$where_sql .= " CAST(concat(logs.booking_date, ' ', logs.booking_time) as datetime) ";
				$where_sql .= $wpdb->prepare( 'between %s AND %s ', $date_from, $date_to );
			}
		}


		if ( $where_sql == '' ) {
			$where_sql = '1';
		}


		$sortingOrder = " ORDER BY $orderby $order ";

		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		$count = $wpdb->get_var( "$sql_select $join  WHERE  $where_sql $sortingOrder" );

		return $count;
	}//end getLogDataCount


	/**
	 * Get the top links(Time based items) before the table listing
	 *
	 * @return array
	 */
	public static function get_range_views( $date_range = 'all', $cbxrblogfromDate = '', $cbxrblogtoDate = '' ) {
		$views = [];

		$date_range_url = remove_query_arg( [ 'status', 's' ] );

		//All link
		$class        = ( $date_range == 'all' ? ' class="current"' : '' );
		$all_url      = add_query_arg( 'date_range', 'all', $date_range_url );
		$views['all'] = "<a href='{$all_url }' {$class} >" . esc_html__( 'All', 'cbxrbooking' ) . '</a>';

		//Upcoming link
		$class             = ( $date_range == 'upcoming' ? ' class="current"' : '' );
		$link_url          = add_query_arg( 'date_range', 'upcoming', $date_range_url );
		$views['upcoming'] = "<a href='{$link_url }' {$class} >" . esc_html__( 'Upcoming', 'cbxrbooking' ) . '</a>';

		//Today link
		$class          = ( $date_range == 'today' ? ' class="current"' : '' );
		$link_url       = add_query_arg( 'date_range', 'today', $date_range_url );
		$views['today'] = "<a href='{$link_url }' {$class} >" . esc_html__( 'Today', 'cbxrbooking' ) . '</a>';

		//Previous link
		$class             = ( $date_range == 'previous' ? ' class="current"' : '' );
		$link_url          = add_query_arg( 'date_range', 'previous', $date_range_url );
		$views['previous'] = "<a href='{$link_url }' {$class} >" . esc_html__( 'Previous', 'cbxrbooking' ) . '</a>';

		//Between dates link
		$class    = ( $date_range == 'between_dates' ? ' class="cbx-between-dates-link current"' : ' class="cbx-between-dates-link"' );
		$link_url = add_query_arg( 'date_range', 'between_dates', $date_range_url );

		if ( $date_range == 'between_dates' && $cbxrblogfromDate != '' && $cbxrblogtoDate != '' ) {
			$link_url = add_query_arg( 'cbxrblogfromDate', $cbxrblogfromDate, $link_url );
			$link_url = add_query_arg( 'cbxrblogtoDate', $cbxrblogtoDate, $link_url );

			$views['between_dates'] = "<a href='{$link_url }' {$class} >" . esc_html__( 'Between Dates',
					'cbxrbooking' ) . ' (' . $cbxrblogfromDate . esc_html__( ' - ',
					'cbxrbooking' ) . $cbxrblogtoDate . ')' . '</a>';
		} else {
			$views['between_dates'] = "<a href='{$link_url }' {$class} >" . esc_html__( 'Between Dates',
					'cbxrbooking' ) . '</a>';
		}

		return $views;
	}//end get_range_views


	/**
	 * Get the top links(Status based items based on time filter) before the table listing
	 *
	 * @return array
	 */
	public static function get_views_status() {
		$views = [];

		$date_range = ( isset( $_REQUEST['date_range'] ) ? sanitize_text_field( wp_unslash($_REQUEST['date_range'])) : 'all' ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		$current    = ( isset( $_REQUEST['status'] ) ? sanitize_text_field( wp_unslash($_REQUEST['status'])) : 'all' ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended 

		$status_arr = self::getAllBookingStatus();

		$count_by_status = self::getCountByStatus();

		$change_url = remove_query_arg( [ 's' ] );

		//All link
		$class        = ( $current == 'all' ? ' class="current"' : '' );
		$all_url      = add_query_arg( 'status', 'all', $change_url );
		/* translators: %d: count  */
		$views['all'] = "<a href='{$all_url }' {$class} >" . sprintf( esc_html__( 'All (%d)', 'cbxrbooking' ),
				$count_by_status['total'] ) . '</a>';

		foreach ( $status_arr as $key => $value ) {
			$url           = add_query_arg( 'status', $key, $change_url );
			$class         = ( $current == $key ? ' class="current"' : '' );
			/* translators: %1$s: status value , %2$d: status count  */
			$views[ $key ] = "<a href='{$url}' {$class} >" . sprintf( esc_html__( '%1$s (%2$d)', 'cbxrbooking' ), esc_html($value),
					$count_by_status[ $key ] ) . '</a>';
		}

		return $views;
	}//end get_views_status


	/**
	 * Return booking count respectively status
	 *
	 * @return array
	 */
	public static function getCountByStatus() {

		global $wpdb;

		$current_offset = get_option( 'gmt_offset' );
		$tzstring       = get_option( 'timezone_string' );

		$check_zone_info = true;

		// Remove old Etc mappings. Fallback to gmt_offset.
		if ( false !== strpos( $tzstring, 'Etc/GMT' ) ) {
			$tzstring = '';
		}

		if ( empty( $tzstring ) ) { // Create a UTC+- zone if no timezone string exists
			$check_zone_info = false;
			if ( 0 == $current_offset ) {
				$tzstring = '+0';
			} elseif ( $current_offset < 0 ) {
				$tzstring = '' . $current_offset;
			} else {
				$tzstring = '+' . $current_offset;
			}
		}

		$date_time_zone           = new DateTimeZone( $tzstring );
		$today_datetime_requested = new DateTime( 'now', $date_time_zone );
		$today_date               = $today_datetime_requested->format( 'Y-m-d' );

		$booking_table = $wpdb->prefix . 'cbxrbooking_log_manager';

		$where_sql = '';

		$join = '';
		if ( isset( $_REQUEST['branch_id'] ) && intval( $_REQUEST['branch_id'] ) > 0 ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended 

			$postmeta_table = $wpdb->prefix . 'postmeta'; // postmeta

			$join .= " JOIN $postmeta_table postmeta ON bt.form_id = postmeta.post_id";

			$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
			$where_sql .= $wpdb->prepare( 'postmeta.meta_key = %s AND postmeta.meta_value = %d',
				'_cbxrbookingformmeta_branch', intval( $_REQUEST['branch_id'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		}


		if ( isset( $_REQUEST['form_id'] ) && intval( $_REQUEST['form_id'] ) > 0 ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
			if ( $where_sql != '' ) {
				$where_sql .= ' AND ';
			}
			$where_sql .= $wpdb->prepare( 'bt.form_id=%d', intval( $_REQUEST['form_id'] ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended 
		}

		$date_range = ( isset( $_REQUEST['date_range'] ) ? sanitize_text_field( wp_unslash($_REQUEST['date_range'])) : 'all' ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended 

		$search = ( isset( $_REQUEST['s'] ) && $_REQUEST['s'] != '' ) ? sanitize_text_field( wp_unslash($_REQUEST['s'] )) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Recommended      
		if ( $search != '' ) {
			if ( $where_sql != '' ) {
				$where_sql .= ' AND ';
			}
			$search_like = '%' . $wpdb->esc_like( $search ) . '%';
			$where_sql .= $wpdb->prepare( " name LIKE %s OR email LIKE %s OR phone LIKE %s OR message LIKE %s OR secret LIKE %s",
				$search_like, $search_like, $search_like, $search_like, $search_like );
		}

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$date_from = isset( $_REQUEST['cbxrblogfromDate'] ) ? sanitize_text_field(wp_unslash($_REQUEST['cbxrblogfromDate'])) : ''; //date from
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$date_to   = isset( $_REQUEST['cbxrblogtoDate'] ) ? sanitize_text_field(wp_unslash($_REQUEST['cbxrblogtoDate'])) : ''; //date end

		if ( $date_range !== '' ) {
			if ( $date_range === 'upcoming' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= $wpdb->prepare( 'booking_date>%s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date > %s', $today_date );
			} elseif ( $date_range === 'today' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= $wpdb->prepare( 'booking_date=%s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date = %s', $today_date );
			} elseif ( $date_range === 'previous' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );

				//$where_sql .= $wpdb->prepare( 'booking_date<%s', date( "Y-m-d" ) );
				$where_sql .= $wpdb->prepare( 'booking_date < %s', $today_date );

			} elseif ( $date_range === 'between_dates' && $date_from != '' && $date_to != '' ) {
				$where_sql .= ( ( $where_sql != '' ) ? ' AND ' : '' );
				//$where_sql .= " CAST(bt.booking_date AS DATE) ";
				$where_sql .= " CAST(concat(bt.booking_date, ' ', bt.booking_time) as datetime) ";
				$where_sql .= $wpdb->prepare( 'between %s AND %s ', $date_from, $date_to );
			}
		}

		if ( $where_sql === '' ) {
			$where_sql = '1';
		}


		$sql_select = "SELECT booking_date, status, COUNT(*) as status_counts FROM $booking_table as bt $join WHERE  $where_sql GROUP BY status";
		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		$results = $wpdb->get_results( "$sql_select", 'ARRAY_A' );

		$total           = 0;
		$count_by_status = self::getAllBookingStatus();
		foreach ( $count_by_status as $key => $value ) {
			$count_by_status[ $key ] = 0;
		}
		$count_by_status['total'] = $total;


		if ( $results != null ) {
			foreach ( $results as $result ) {
				$total                                += intval( $result['status_counts'] );
				$count_by_status[ $result['status'] ] = $result['status_counts'];
			}
			$count_by_status['total'] = $total;
		}

		return $count_by_status;
	}//end getCountByStatus

	/**
	 * am pm hour format
	 *
	 * @param  string  $timestamp
	 *
	 * @return false|string
	 */
	public static function ampmTimeFormat( $timestamp = '' ) {
		return gmdate( 'h:ia', strtotime( $timestamp ) );
	}//end ampmTimeFormat


	/**
	 * form setting time format
	 *
	 * @param  string  $timestamp
	 *
	 * @return false|string
	 */
	public static function twelveHourBookingTimeFormat( $post_id = 0, $booking_time = '' ) {
		$twelve_hour_booking_time = null;

		$post_id = intval( $post_id );
		if ( $booking_time != '' ) {
			if ( $post_id > 0 ) {
				$cbxrbooking_meta = get_post_meta( $post_id, '_cbxrbookingmeta', true );
				if ( is_array( $cbxrbooking_meta ) && sizeof( $cbxrbooking_meta ) > 0 ) {
					if ( isset( $cbxrbooking_meta['settings']['cbxrbooking_booking_schedule']['time_format'] ) ) {
						$time_format = intval( $cbxrbooking_meta['settings']['cbxrbooking_booking_schedule']['time_format'] );
						if ( $time_format == 12 ) {
							$twelve_hour_booking_time = self::ampmTimeFormat( $booking_time );
						}
					}
				}
			} else {
				$cbxrbooking_global_setting = get_option( 'cbxrbooking_global', null );
				if ( ! is_null( $cbxrbooking_global_setting ) ) {
					if ( isset( $cbxrbooking_global_setting['time_format'] ) ) {
						$time_format = intval( $cbxrbooking_global_setting['time_format'] );
						if ( $time_format == 12 ) {
							$twelve_hour_booking_time = self::ampmTimeFormat( $booking_time );
						}
					}
				}
			}
		}

		return $twelve_hour_booking_time;
	}//end twelveHourBookingTimeFormat

	/**
	 * Get the default state dropdown
	 *
	 * @return array
	 */
	public static function get_default_states() {
		$booking_states = CBXRBookingHelper::getAllBookingStatus();
		unset( $booking_states['unverified'] );
		unset( $booking_states['canceled'] );
		unset( $booking_states['waiting'] );
		unset( $booking_states['seated'] );
		unset( $booking_states['departed'] );
		unset( $booking_states['closed'] );

		return apply_filters( 'cbxrbooking_defaultstate', $booking_states );
	}//end get_default_states

	/**
	 * Checking date validity
	 *
	 * @param  string  $date
	 * @param  string  $format
	 *
	 * @return bool
	 */
	public static function validateDate( $date, $format = 'Y-m-d H:i:s' ) {
		$d = DateTime::createFromFormat( $format, $date );

		return $d && $d->format( $format ) == $date;
	}//end validateDate

	/**
	 * Get the early bookings dropdown
	 *
	 * @return array
	 */
	public static function get_early_bookings() {
		$early_bookings = [
			''   => esc_html__( 'Any Time', 'cbxrbooking' ),
			'1'  => esc_html__( 'From 1 day in advance', 'cbxrbooking' ),
			'7'  => esc_html__( 'From 1 week in advance', 'cbxrbooking' ),
			'14' => esc_html__( 'From 2 weeks in advance', 'cbxrbooking' ),
			'30' => esc_html__( 'From 30 days in advance', 'cbxrbooking' ),
			'90' => esc_html__( 'From 90 days in advance', 'cbxrbooking' ),
		];

		return apply_filters( 'cbxrbooking_earlybookings', $early_bookings );
	}//end get_early_bookings

	/**
	 * Get early booking message by selected value
	 *
	 * @param  string  $value
	 *
	 * @return string
	 */
	public static function getEarlyBookingByValue( $value = '' ) {

		if ( $value == '' ) {
			return '';
		}

		$early_bookings = CBXRBookingHelper::get_early_bookings();

		if ( isset( $early_bookings[ $value ] ) ) {
			return $early_bookings[ $value ];
		} else {
			return '';
		}
	}//end getEarlyBookingByValue

	/**
	 * Get the late bookings dropdown
	 *
	 * @return array
	 */
	public static function get_late_bookings() {
		$late_bookings = [
			''         => esc_html__( 'Up to the last minute', 'cbxrbooking' ),
			'15'       => esc_html__( 'At least 15 minutes in advance', 'cbxrbooking' ),
			'30'       => esc_html__( 'At least 30 minutes in advance', 'cbxrbooking' ),
			'45'       => esc_html__( 'At least 45 minutes in advance', 'cbxrbooking' ),
			'60'       => esc_html__( 'At least 1 hour in advance', 'cbxrbooking' ),
			'240'      => esc_html__( 'At least 4 hours in advance', 'cbxrbooking' ),
			'1440'     => esc_html__( 'At least 24 hours in advance', 'cbxrbooking' ),
			'same_day' => esc_html__( 'Block same-day bookings', 'cbxrbooking' ),
		];

		return apply_filters( 'cbxrbooking_latebookings', $late_bookings );
	}//end get_late_bookings

	/**
	 * Get late booking message by selected value
	 *
	 * @param  string  $value
	 *
	 * @return string
	 */
	public static function getLateBookingByValue( $value = '' ) {

		if ( $value == '' ) {
			return '';
		}

		$late_bookings = CBXRBookingHelper::get_late_bookings();

		if ( isset( $late_bookings[ $value ] ) ) {
			return $late_bookings[ $value ];
		} else {
			return '';
		}
	}//end getLateBookingByValue

	/**
	 * Get the time dropdown
	 *
	 * @return array
	 */
	public static function get_time_interval() {
		$booking_time_interval = [
			'30' => esc_html__( 'Every 30 minutes', 'cbxrbooking' ),
			'15' => esc_html__( 'Every 15 minutes', 'cbxrbooking' ),
			'10' => esc_html__( 'Every 10 minutes', 'cbxrbooking' ),
			'5'  => esc_html__( 'Every 5 minutes', 'cbxrbooking' ),
		];

		return apply_filters( 'cbxrbooking_gettimeinterval', $booking_time_interval );
	}

	/**
	 * Get the week starts on dropdown
	 *
	 * @return array
	 */
	public static function get_week_starts_on() {
		$booking_week_starts_on = [
			'0' => __( 'Sunday', 'cbxrbooking' ),
			'1' => __( 'Monday', 'cbxrbooking' ),
			'2' => __( 'Tuesday', 'cbxrbooking' ),
			'3' => __( 'Wednesday', 'cbxrbooking' ),
			'4' => __( 'Thursday', 'cbxrbooking' ),
			'5' => __( 'Friday', 'cbxrbooking' ),
			'6' => __( 'Saturday', 'cbxrbooking' ),
		];

		return $booking_week_starts_on;
	}//end get_week_starts_on

	/**
	 * Get the early bookings cancel dropdown
	 *
	 * @return mixed|void
	 */
	public static function get_early_cancels() {
		$booking_cancel = [
			''         => esc_html__( 'Any Time', 'cbxrbooking' ),
			'15'       => esc_html__( 'At least 15 minutes in advance', 'cbxrbooking' ),
			'30'       => esc_html__( 'At least 30 minutes in advance', 'cbxrbooking' ),
			'45'       => esc_html__( 'At least 45 minutes in advance', 'cbxrbooking' ),
			'60'       => esc_html__( 'At least 1 hour in advance', 'cbxrbooking' ),
			'240'      => esc_html__( 'At least 4 hours in advance', 'cbxrbooking' ),
			'1440'     => esc_html__( 'At least 24 hours in advance', 'cbxrbooking' ),
			'same_day' => esc_html__( 'Block same-day Cancel', 'cbxrbooking' ),
		];

		return apply_filters( 'cbxrbooking_cancelbookings', $booking_cancel );
	}//end get_early_cancels

	/**
	 *  get list of post statuses
	 *
	 * @return mixed|void
	 */
	public static function form_statuses() {
		return [
			'publish'    => 'Published',         // The post is publicly visible.
			'draft'      => 'Draft',             // The post is in draft mode and not yet published.
			'pending'    => 'Pending Review',    // The post is awaiting approval or review.
			'private'    => 'Private',           // The post is private and visible only to site admins and editors.
			'future'     => 'Scheduled',         // The post is scheduled to be published in the future.
			'trash'      => 'Trash',             // The post has been moved to the trash.
			'auto-draft' => 'Auto Draft',        // The post is an automatically saved draft.
			'inherit'    => 'Inherited',         // Used for revisions or attachments, inherits status from the parent.
			'protected'  => 'Protected'          // A password-protected post.
		];
	}//end form_statuses

	/**
	 * Setup a post object and store the original loop item so we can reset it later
	 *
	 * @param  obj  $post_to_setup  The post that we want to use from our custom loop
	 */
	public static function setup_postdata( $post_to_setup ) {

		//only on the admin side
		if ( is_admin() ) {

			//get the post for both setup_postdata() and to be cached
			global $post;

			//only cache $post the first time through the loop
			if ( ! isset( $GLOBALS['post_cache'] ) ) {
				$GLOBALS['post_cache'] = $post;
			}

			//setup the post data as usual
			$post = $post_to_setup;
			setup_postdata( $post );
		} else {
			setup_postdata( $post_to_setup );
		}
	}//end setup_postdata


	/**
	 * Reset $post back to the original item
	 *
	 */
	public static function wp_reset_postdata() {

		//only on the admin and if post_cache is set
		if ( is_admin() && ! empty( $GLOBALS['post_cache'] ) ) {

			//globalize post as usual
			global $post;

			//set $post back to the cached version and set it up
			$post = $GLOBALS['post_cache'];
			setup_postdata( $post );

			//cleanup
			unset( $GLOBALS['post_cache'] );
		} else {
			wp_reset_postdata();
		}
	}//end wp_reset_postdata

	/**
	 * Create tables as need
	 */
	public static function create_table() {
		global $wpdb;

		//old name
		$table_booking_logs_old   = $wpdb->prefix . 'rbookinglogs';
		$table_branch_manager_old = $wpdb->prefix . 'branch_manager';


		//new name
		$table_booking_logs   = $wpdb->prefix . 'cbxrbooking_log_manager';
		$table_branch_manager = $wpdb->prefix . 'cbxrbooking_branch_manager';


		//start name process
		//rename log table
		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_booking_logs_old'" ) === $table_booking_logs_old ) {
			//Your code here
			// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
			$wpdb->query( "RENAME TABLE $table_booking_logs_old TO $table_booking_logs" );
		}

		//rename branch manager table
		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
		if ( $wpdb->get_var( "SHOW TABLES LIKE '$table_branch_manager_old'" ) === $table_branch_manager_old ) {
			//Your code here
			// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
			$wpdb->query( "RENAME TABLE $table_branch_manager_old TO $table_branch_manager" );
		}
		//end rename process


		//db table migration if exists
		$charset_collate = '';
		if ( $wpdb->has_cap( 'collation' ) ) {
			if ( ! empty( $wpdb->charset ) ) {
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$charset_collate .= " COLLATE $wpdb->collate";
			}
		}


		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

		//create rbookinglog table
		$table_rbookings_sql = "CREATE TABLE $table_booking_logs (
						id bigint(11) unsigned NOT NULL AUTO_INCREMENT,
						form_id bigint(11) unsigned NOT NULL DEFAULT 0 COMMENT 'booking form id of custom post type',
						name varchar(255) NOT NULL DEFAULT '' COMMENT 'client name',
						email varchar(100) NOT NULL DEFAULT '' COMMENT 'client email',
						phone varchar(25) DEFAULT '' COMMENT 'client phone number',
						booking_date DATE NOT NULL COMMENT 'client preferred booking date',
						booking_time TIME NOT NULL COMMENT 'client preferred booking time',
						party_size mediumint(3) unsigned NOT NULL DEFAULT '0' COMMENT 'client preferred party size',
						message text DEFAULT '' COMMENT 'Client Message',
						user_ip varchar(45) NOT NULL,
						add_by bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'foreign key of user table. who added this, if uest zero',
						mod_by bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'foreign key of user table. who last modify this list',
						add_date datetime DEFAULT NULL COMMENT 'add date',
						mod_date datetime DEFAULT NULL COMMENT 'last modified date',
						status VARCHAR (255) NOT NULL DEFAULT 'pending',
						secret VARCHAR(255) NOT NULL DEFAULT '' COMMENT 'booking secret code',                          
						activation VARCHAR(255) DEFAULT NULL COMMENT 'activation code',                          
						metadata text DEFAULT '' COMMENT 'any extra information needed information',                          
						PRIMARY KEY  (id)
					) $charset_collate; ";


		$table_cbxrb_branch_manager_sql = "CREATE TABLE $table_branch_manager (
						id int(11) unsigned NOT NULL AUTO_INCREMENT,
						name varchar(200) NOT NULL COMMENT 'branch manager name',
						description varchar(200) DEFAULT NULL COMMENT 'branch manager description',
						address text DEFAULT NULL COMMENT 'branch manager address',
						add_by bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'foreign key of user table. who add this branch manager',
						mod_by bigint(20) unsigned NOT NULL DEFAULT '0' COMMENT 'foreign key of user table. who modified this branch manager',
						add_date datetime DEFAULT NULL COMMENT 'created date',
						mod_date datetime DEFAULT NULL COMMENT 'modified date',
						PRIMARY KEY  (id)
					) $charset_collate; ";

		dbDelta( [ $table_rbookings_sql, $table_cbxrb_branch_manager_sql ] );
	}


	/**
	 * Is gutenberg edit page
	 *
	 * @return bool
	 */
	public static function is_gutenberg_page() {
		//if(!is_admin()) return false;
		if ( function_exists( 'is_gutenberg_page' ) &&
		     is_gutenberg_page()
		) {
			// The Gutenberg plugin is on.
			return true;
		}

		$current_screen = get_current_screen();
		if ( method_exists( $current_screen, 'is_block_editor' ) && $current_screen->is_block_editor() ) {
			// Gutenberg page on 5+.
			return true;
		}

		return false;
	}//end is_gutenberg_page

	/**
	 * Add utm params to any url
	 *
	 * @param  string  $url
	 *
	 * @return string
	 */
	public static function url_utmy( $url = '' ) {
		if ( $url == '' ) {
			return $url;
		}

		$url = add_query_arg( [
			'utm_source'   => 'plgdashboardinfo',
			'utm_medium'   => 'plgdashboard',
			'utm_campaign' => 'wpfreemium',
		], $url );

		return $url;
	}//end url_utmy

	/**
	 * dashboard menu list
	 *
	 * @since 1.0.0
	 */
	public static function dashboard_menus() {
		$menus = [];

		if ( current_user_can( 'cbxrbooking_form_manage' ) ) {
			$menus['cbxrbooking-form'] = [
				'url'   => admin_url( 'admin.php?page=cbxrbooking-form' ),
				'title' => esc_html__( 'Booking Forms', 'cbxrbooking' ),
				'label' => esc_html__( 'Forms Manager', 'cbxrbooking' ),
			];
		}

		if ( current_user_can( 'cbxrbooking_log_manage' ) ) {
			$menus['cbxrbooking-log'] = [
				'url'   => admin_url( 'admin.php?page=cbxrbookinglogs' ),
				'title' => esc_html__( 'Booking Logs', 'cbxrbooking' ),
				'label' => esc_html__( 'Logs Manager', 'cbxrbooking' ),
			];
		}

		if ( current_user_can( 'cbxrbooking_branch_manage' ) ) {
			$menus['cbxrbooking-branch'] = [
				'url'   => admin_url( 'admin.php?page=cbxrbookingbranchmanager' ),
				'title' => esc_html__( 'Branches', 'cbxrbooking' ),
				'label' => esc_html__( 'Branch Manager', 'cbxrbooking' ),
			];
		}


		if ( current_user_can( 'cbxrbooking_settings_manage' ) ) {
			$menus['cbxrbooking_settings'] = [
				'url'   => admin_url( 'admin.php?page=cbxrbookingsettings' ),
				'title' => esc_attr__( 'Global Settings', 'cbxrbooking' ),
				'label' => esc_html__( 'Global Settings', 'cbxrbooking' ),
			];
		}

		if ( current_user_can( 'cbxrbooking_settings_manage' ) ) {
			$menus['cbxrbooking_support'] = [
				'url'   => admin_url( 'admin.php?page=cbxrbooking-help-support' ),
				'title' => htmlspecialchars_decode( esc_attr__( 'Helps & Updates', 'cbxrbooking' ) ),
				'label' => htmlspecialchars_decode( esc_html__( 'Helps & Updates', 'cbxrbooking' ) ),
			];
		}

		return $menus;
	}// end function dashboard_menus

	/**
	 * common js translation and variable used in plugin
	 *
	 * @param  string  $current_user
	 * @param  string  $blog_id
	 *
	 * @return mixed|void
	 * @since 1.0.0
	 */
	public static function common_js_translation( $current_user = '', $blog_id = '' ) {
		$current_user = ( $current_user == '' ) ? wp_get_current_user() : $current_user;

		if ( $blog_id == '' ) {
			$blog_id = is_multisite() ? get_current_blog_id() : null;
		}

		$js_translations = [
			'icons_url'             => CBXRBOOKING_ROOT_URL . '/assets/icons/',
			'nonce'                 => wp_create_nonce( 'cbxrbooking' ),
			'rest_nonce'            => wp_create_nonce( 'wp_rest' ),
			'dashboard_menus'       => self::dashboard_menus(),
			'site_url'              => site_url(),
			'admin_url'             => admin_url(),
			'translations'          => [
				'copy_labels'               => [
					'copy_before' => esc_html__( 'Copy', 'cbxrbooking' ),
					'copy_after'  => esc_html__( 'Copied', 'cbxrbooking' )
				],
				'buttons'                   => [
					'close'  => [
						'title'    => esc_attr__( 'Click to close', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Close', 'cbxrbooking' )
					],
					'search' => [
						'title'    => esc_attr__( 'Click to search', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Search', 'cbxrbooking' )
					],
					'reset'  => [
						'title'    => esc_attr__( 'Click to reset', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Reset', 'cbxrbooking' )
					],
					'filter' => [
						'title'    => esc_attr__( 'Column Filter', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Filter', 'cbxrbooking' )
					],
					'view'   => [
						'title'    => esc_attr__( 'Click to view', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'View', 'cbxrbooking' )
					],
					'clone'  => [
						'title'    => esc_attr__( 'Click to clone', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Clone', 'cbxrbooking' )
					],
					'edit'   => [
						'title'    => esc_attr__( 'Click to edit', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Edit', 'cbxrbooking' )
					],
					'delete' => [
						'title'    => esc_attr__( 'Click to delete', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Delete', 'cbxrbooking' )
					],
					'gear'   => [
						'title'    => esc_attr__( 'Click to Configure', 'cbxrbooking' ),
						'sr_label' => esc_html__( 'Configure', 'cbxrbooking' )
					],
				],
				'add'                       => esc_html__( 'Add', 'cbxrbooking' ),
				'delete_all'                => esc_html__( 'Delete All', 'cbxrbooking' ),
				'total'                     => esc_html__( 'Total', 'cbxrbooking' ),
				'id'                        => esc_html__( 'ID', 'cbxrbooking' ),
				'action'                    => esc_html__( 'Action', 'cbxrbooking' ),
				'delete_confirmation_title' => esc_html__( 'Are you sure?', 'cbxrbooking' ),
				'delete_confirmation_txt'   => esc_html__( 'You want to delete this.', 'cbxrbooking' ),
				'delete_btn_txt'            => esc_html__( 'Delete', 'cbxrbooking' ),
				'showing'                   => esc_html__( 'Showing ', 'cbxrbooking' ),
				'of'                        => esc_html__( 'of', 'cbxrbooking' ),
				'rowCount'                  => esc_html__( 'Row count ', 'cbxrbooking' ),
				'goTo'                      => esc_html__( 'Go to page ', 'cbxrbooking' ),
				'delete'                    => esc_html__( 'Delete', 'cbxrbooking' ),
				'status'                    => esc_html__( 'Status', 'cbxrbooking' ),
				'attachment'                => esc_html__( 'Attachment', 'cbxrbooking' ),
				'upload'                    => esc_html__( 'Upload', 'cbxrbooking' ),
				'posted'                    => esc_html__( 'Posted', 'cbxrbooking' ),
				'back'                      => esc_html__( 'Back', 'cbxrbooking' ),
				'save'                      => esc_html__( 'Save', 'cbxrbooking' ),
				'update'                    => esc_html__( 'Update', 'cbxrbooking' ),
				'search_text'               => esc_html__( 'Search...', 'cbxrbooking' ),
			],
			'rest_end_points'       => [
				'get_form'               => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/form-data' ) ),
				'create_form'            => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/create-form' ) ),
				'save_form'              => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/save-form' ) ),
				'get_form_list'          => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/form-list' ) ),
				'delete_form'            => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/delete-form' ) ),
				'reset_submission_count' => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/reset-submission-count' ) ),

				'get_log'       => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/log-data' ) ),
				'save_log'      => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/save-log' ) ),
				'get_log_list'  => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/log-list' ) ),
				'delete_log'    => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/delete-log' ) ),
				'change_status' => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/change-booking-status' ) ),

				'get_branch'      => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/branch-data' ) ),
				'save_branch'     => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/save-branch' ) ),
				'get_branch_list' => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/branch-list' ) ),
				'delete_branch'   => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/delete-branch' ) ),

				'reset_option'       => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/reset-option' ) ),
				'migrate_table'      => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/migrate-table' ) ),
				'dashboard_overview' => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/dashboard-overview' ) ),
				'dashboard_listing'  => esc_url_raw( get_rest_url( $blog_id, 'cbxrbooking/v1/dashboard-listing' ) ),
			],
			'cbx_table_lite'        => self::table_light_translation(),
			'cbxrbookingpro_active' => function_exists( 'cbxrbooking_check_pro_plugin_active' ) && cbxrbooking_check_pro_plugin_active(),
		];

		return $js_translations;
	} //end method common_js_translation

	/**
	 * log builder js translation list
	 *
	 * @param $current_user
	 * @param $blog_id
	 *
	 * @return mixed|void
	 */
	public static function cbxrbooking_log_builder_js_translation() {
		$common_js_translations = self::common_js_translation();

		$branch_list = RBranch::get()->pluck( 'name', 'id' )->toArray();
		$form_list   = Post::bookingForms()->get()->pluck( 'post_title', 'ID' )->toArray();

		$form_js_translations = [
			'translations'       => [
				'app_heading'                 => esc_html__( 'Restaurant Booking : Log Manager', 'cbxrbooking' ),
				'add_new_booking'             => esc_html__( 'Add New Booking', 'cbxrbooking' ),
				'booking_calendar_view'       => esc_html__( 'Booking Calendar View', 'cbxrbooking' ),
				'choose_booking_form'         => esc_html__( 'Choose booking form', 'cbxrbooking' ),
				'no_booking_found'            => esc_html__( 'No booking found', 'cbxrbooking' ),
				'search_log'                  => esc_html__( 'Search Log', 'cbxrbooking' ),
				'bookings'                    => esc_html__( 'Bookings', 'cbxrbooking' ),
				'add_booking'                 => esc_html__( 'Add Booking', 'cbxrbooking' ),
				'edit_booking'                => esc_html__( 'Edit Booking', 'cbxrbooking' ),
				'name'                        => esc_html__( 'Name', 'cbxrbooking' ),
				'email'                       => esc_html__( 'Email', 'cbxrbooking' ),
				'date'                        => esc_html__( 'Date', 'cbxrbooking' ),
				'time'                        => esc_html__( 'Time', 'cbxrbooking' ),
				'phone'                       => esc_html__( 'Phone', 'cbxrbooking' ),
				'message'                     => esc_html__( 'Message', 'cbxrbooking' ),
				'party_size'                  => esc_html__( 'Party Size', 'cbxrbooking' ),
				'code'                        => esc_html__( 'Code', 'cbxrbooking' ),
				'form'                        => esc_html__( 'Form', 'cbxrbooking' ),
				'size'                        => esc_html__( 'Size', 'cbxrbooking' ),
				'ip'                          => esc_html__( 'IP', 'cbxrbooking' ),
				'notify_user_by_email'        => esc_html__( 'Notify User by Email', 'cbxrbooking' ),
				'select_status'               => esc_html__( 'Select status', 'cbxrbooking' ),
				'date_range'                  => esc_html__( 'Date Range', 'cbxrbooking' ),
				'user_notification'           => esc_html__( 'User notification', 'cbxrbooking' ),
				'booking_code'                => esc_html__( 'Booking code', 'cbxrbooking' ),
				'select_branch'               => esc_html__( 'Select Branch', 'cbxrbooking' ),
				'select_form'                 => esc_html__( 'Select form', 'cbxrbooking' ),
				'switch_calendar_view'        => esc_html__( 'Switch to calendar view', 'cbxrbooking' ),
				'change_to'                   => esc_html__( 'Change to ', 'cbxrbooking' ),
				'export'                      => esc_html__( 'Export', 'cbxrbooking' ),
				'notice'                      => esc_html__( 'Notice', 'cbxrbooking' ),
				'spreadsheet_not_active'      => esc_html__( 'CBXPhpSpreadsheet Library is not active.Please intall and try again.', 'cbxrbooking' ),
				'cbxphpspreadsheet_not_found' => wp_kses( __( 'For export needs PHP spreadsheet library. Please download the <a href="https://codeboxr.com/php-spreadsheet-library-wordpress-plugin/" target="_blank">PHP Spreadsheet Library</a> based on this documentation, install and active.', 'cbxrbooking' ), [ 'a' => [ 'href' => [], 'target' => [], 'class' => [] ] ] )
			],
			'exportnonce'        => wp_create_nonce( 'cbxrb_export' ),
			'publishedFroms'     => self::publishedFroms(),
			'booking_statuses'   => self::getAllBookingStatus(),
			'branch_list'        => $branch_list,
			'form_list'          => $form_list,
			'spreadsheet_active' => self::cbxphpspreadsheet_active()
		];

		$js_translations = array_merge_recursive( $common_js_translations, $form_js_translations );

		return apply_filters( 'cbxrbooking_log_js_translation', $js_translations );
	} //end method cbxrbooking_log_builder_js_translation

	/**
	 * branch manager builder js translation list
	 *
	 * @param $current_user
	 * @param $blog_id
	 *
	 * @return mixed|void
	 */
	public static function cbxrbooking_branch_builder_js_translation() {
		$common_js_translations = self::common_js_translation();

		$form_js_translations = [
			'translations'     => [
				'app_heading'           => esc_html__( 'Restaurant Booking : Branches', 'cbxrbooking' ),
				'add_new'               => esc_html__( 'Add New', 'cbxrbooking' ),
				'no_manager_found'      => esc_html__( 'No Manager found', 'cbxrbooking' ),
				'branches'              => esc_html__( 'Branches', 'cbxrbooking' ),
				'add_branch'            => esc_html__( 'Add Branch', 'cbxrbooking' ),
				'edit_branch'           => esc_html__( 'Edit Branch', 'cbxrbooking' ),
				'branch_name'           => esc_html__( 'Branch Name', 'cbxrbooking' ),
				'description'           => esc_html__( 'Description', 'cbxrbooking' ),
				'address'               => esc_html__( 'Address', 'cbxrbooking' ),
				'address_line'          => esc_html__( 'Address Line', 'cbxrbooking' ),
				'city'                  => esc_html__( 'City', 'cbxrbooking' ),
				'state_province_region' => esc_html__( 'State/Province/Region', 'cbxrbooking' ),
				'state'                 => esc_html__( 'State', 'cbxrbooking' ),
				'zip_postal_code'       => esc_html__( 'Zip/Postal Code', 'cbxrbooking' ),
				'postal'                => esc_html__( 'Postal', 'cbxrbooking' ),
				'country'               => esc_html__( 'Country', 'cbxrbooking' ),
				'select_country'        => esc_html__( 'Select Country', 'cbxrbooking' ),

				// ''                     => esc_html__( '', 'cbxrbooking' ),
			],
			'publishedFroms'   => self::publishedFroms(),
			'booking_statuses' => self::getAllBookingStatus(),
			'country_list'     => self::getAllCountries(),
			'branch_count'     => RBranch::count()
		];

		$js_translations = array_merge_recursive( $common_js_translations, $form_js_translations );

		return apply_filters( 'cbxrbooking_log_js_translation', $js_translations );
	} //end method cbxrbooking_branch_builder_js_translation

	/**
	 * form builder js translation list
	 *
	 * @param $current_user
	 * @param $blog_id
	 *
	 * @return mixed|void
	 */
	public static function cbxrbooking_form_builder_js_translation() {
		$common_js_translations = self::common_js_translation();

		$admin         = new CBXRBookingAdmin();
		$form_sections = $admin->cbxrbookingform_meta_settings_sections();

		$branch_list = RBranch::get()->pluck( 'name', 'id' )->toArray();

		$form_js_translations = [
			'translations'   => [
				'export'              => esc_html__( 'Export', 'cbxrbooking' ),
				'import'              => esc_html__( 'Import', 'cbxrbooking' ),
				'export_form'         => esc_html__( 'Export form', 'cbxrbooking' ),
				'import_form'         => esc_html__( 'Import form', 'cbxrbooking' ),
				'upload_replace'      => esc_html__( 'Upload and Replace', 'cbxrbooking' ),
				'form_import'         => esc_html__( 'Form Import', 'cbxrbooking' ),
				'import_json'         => esc_html__( 'Import Json', 'cbxrbooking' ),
				'app_heading'         => esc_html__( 'Restaurant Booking : Forms', 'cbxrbooking' ),
				'add_new'             => esc_html__( 'Add New', 'cbxrbooking' ),
				'no_form_found'       => esc_html__( 'No Form found', 'cbxrbooking' ),
				'forms'               => esc_html__( 'Forms', 'cbxrbooking' ),
				'add_form'            => esc_html__( 'Add Form', 'cbxrbooking' ),
				'edit_form'           => esc_html__( 'Edit Form', 'cbxrbooking' ),
				'shortcode'           => esc_html__( 'Shortcode', 'cbxrbooking' ),
				'copy'                => esc_html__( 'Copy', 'cbxrbooking' ),
				'branch_name'         => esc_html__( 'Branch Name', 'cbxrbooking' ),
				'submission_count'    => esc_html__( 'Submission Count', 'cbxrbooking' ),
				'general'             => esc_html__( 'General', 'cbxrbooking' ),
				'schedule'            => esc_html__( 'Schedule', 'cbxrbooking' ),
				'admin_alert'         => esc_html__( 'Admin Alert', 'cbxrbooking' ),
				'user_alert'          => esc_html__( 'User Alert', 'cbxrbooking' ),
				'booking_cancel'      => esc_html__( 'Booking Cancel', 'cbxrbooking' ),
				'misc'                => esc_html__( 'Misc', 'cbxrbooking' ),
				'min_party_size'      => esc_html__( 'Min Party Size', 'cbxrbooking' ),
				'min_party_size_desc' => esc_html__( 'Select minimum party size for bookings.', 'cbxrbooking' ),
				'max_party_size_desc' => esc_html__( 'Select maximum party size for bookings.', 'cbxrbooking' ),
				'max_party_size'      => esc_html__( 'Max Party Size', 'cbxrbooking' ),
				'require_name'        => esc_html__( 'Require Name', 'cbxrbooking' ),
				'require_name_desc'   => esc_html__( 'Is Name is mandatory to request a booking', 'cbxrbooking' ),
				'require_email'       => esc_html__( 'Require Email', 'cbxrbooking' ),
				'require_email_desc'  => esc_html__( 'Is Email address is mandatory to request a booking',
					'cbxrbooking' ),
				'require_phone'       => esc_html__( 'Require Phone', 'cbxrbooking' ),
				'yes'                 => esc_html__( 'Yes', 'cbxrbooking' ),
				'no'                  => esc_html__( 'No', 'cbxrbooking' ),
				'success_message'     => esc_html__( 'Success Message', 'cbxrbooking' ),
				'guest_activation'    => esc_html__( 'Guest Email Activation', 'cbxrbooking' ),
				'default_state'       => esc_html__( 'Default Booking Status', 'cbxrbooking' ),
				'banned_email'        => esc_html__( 'Banned Email', 'cbxrbooking' ),
				'banned_ip'           => esc_html__( 'Banned Ip', 'cbxrbooking' ),
				'ip_address'          => esc_html__( 'Ip Address', 'cbxrbooking' ),
				'text_beforeform'     => esc_html__( 'Text Before Form', 'cbxrbooking' ),
				'text_afterform'      => esc_html__( 'Text After Form', 'cbxrbooking' ),
				'show_user_consent'   => esc_html__( 'Show consent checkbox', 'cbxrbooking' ),
				'date_format'         => esc_html__( 'Date Format', 'cbxrbooking' ),
				'time_format'         => esc_html__( 'Time Format', 'cbxrbooking' ),
				'hour'                => esc_html__( 'Hour', 'cbxrbooking' ),
				'week_days_schedule'  => esc_html__( 'Week Days Schedule', 'cbxrbooking' ),
				'days_of_the_week'    => esc_html__( 'Days of the week', 'cbxrbooking' ),

				'start'                   => esc_html__( 'Start', 'cbxrbooking' ),
				'end'                     => esc_html__( 'End', 'cbxrbooking' ),
				'form'                    => esc_html__( 'Form', 'cbxrbooking' ),
				'date'                    => esc_html__( 'Date', 'cbxrbooking' ),
				'opening_time_slots'      => esc_html__( 'Opening Time Slots', 'cbxrbooking' ),
				'all_day_opened'          => esc_html__( 'All Day Opened', 'cbxrbooking' ),
				'all_day_closed'          => esc_html__( 'All Day Closed', 'cbxrbooking' ),
				'specific_day_exceptions' => esc_html__( 'Specific Day Exceptions', 'cbxrbooking' ),
				'add_new_exception'       => esc_html__( 'Add New Exception', 'cbxrbooking' ),

				'status_of_email'              => esc_html__( 'Status of Email', 'cbxrbooking' ),
				'email_format'                 => esc_html__( 'E-mail Format', 'cbxrbooking' ),
				'admin_email_format_desc'      => esc_html__( 'Select the format of the E-mail.', 'cbxrbooking' ),
				'html'                         => esc_html__( 'HTML', 'cbxrbooking' ),
				'plain'                        => esc_html__( 'Plain', 'cbxrbooking' ),
				'to'                           => esc_html__( 'To', 'cbxrbooking' ),
				'to_email'                     => esc_html__( 'To Email', 'cbxrbooking' ),
				'to_email_address'             => esc_html__( 'To Email Address.', 'cbxrbooking' ),
				'reply_to'                     => esc_html__( 'Reply To', 'cbxrbooking' ),
				'admin_reply_to_desc'          => __( 'Reply To Email Address. Syntax available -', 'cbxrbooking' ),
				'email_subject'                => esc_html__( 'Email Subject.', 'cbxrbooking' ),
				'subject'                      => esc_html__( 'Subject.', 'cbxrbooking' ),
				'email_template_heading'       => esc_html__( 'Email Template heading.', 'cbxrbooking' ),
				'heading'                      => esc_html__( 'Heading', 'cbxrbooking' ),
				'body'                         => esc_html__( 'Body', 'cbxrbooking' ),
				'cc'                           => esc_html__( 'CC', 'cbxrbooking' ),
				'bcc'                          => esc_html__( 'BCC', 'cbxrbooking' ),
				'admin_body_desc'              => __( 'Email Body.  Syntax available - ', 'cbxrbooking' ),
				'admin_cc_desc'                => __( 'Email CC, for multiple use comma', 'cbxrbooking' ),
				'admin_bcc_desc'               => __( 'Email BCC, for multiple use comma', 'cbxrbooking' ),
				'user_email_desc'              => __( 'To Email Address. Syntax available -', 'cbxrbooking' ),
				'user_reply_to_desc'           => __( 'Reply To Email Address.', 'cbxrbooking' ),
				'new_booking_user_email_alert' => esc_html__( 'New Booking User Email Alert', 'cbxrbooking' ),
				'new_subject'                  => esc_html__( 'New Request Email Subject', 'cbxrbooking' ),
				'new_subject_desc'             => __( 'Email subject user will receive when they make an initial booking request.',
					'cbxrbooking' ),
				'new_heading'                  => esc_html__( 'New Request Email Heading', 'cbxrbooking' ),
				'new_heading_desc'             => __( 'Email heading user will receive when they make an initial booking request.',
					'cbxrbooking' ),
				'new_body'                     => esc_html__( 'New Request Email Heading', 'cbxrbooking' ),
				'new_body_desc'                => __( 'Email content user will receive when they make an initial booking request. Syntax available -',
					'cbxrbooking' ),

				'confirmed_booking_user_email_alert' => esc_html__( 'Confirmed Booking User Email Alert',
					'cbxrbooking' ),
				'confirmed_subject'                  => esc_html__( 'Booking Request Confirmed Email Subject',
					'cbxrbooking' ),
				'confirmed_subject_desc'             => __( 'Email subject user will receive when admin confirmed booking request.',
					'cbxrbooking' ),
				'confirmed_heading'                  => esc_html__( 'Booking Request Confirmed Email Heading',
					'cbxrbooking' ),
				'confirmed_heading_desc'             => __( 'Email heading user will receive when admin confirmed booking request.',
					'cbxrbooking' ),
				'confirmed_body'                     => esc_html__( 'Booking Request Confirmed Email Body',
					'cbxrbooking' ),
				'confirmed_body_desc'                => __( 'Email content user will receive when admin confirmed booking request. Syntax available -',
					'cbxrbooking' ),


				'booking_cancel_desc'            => esc_html__( 'Enable/Disable capability if guest can cancel booking',
					'cbxrbooking' ),
				'default_cancel_status'          => esc_html__( 'Default Cancel Status', 'cbxrbooking' ),
				'default_cancel_status_desc'     => esc_html__( 'Default status when user cancel a booking',
					'cbxrbooking' ),
				'canceled'                       => esc_html__( 'Cancelled', 'cbxrbooking' ),
				'cancel_request'                 => esc_html__( 'Cancel Request', 'cbxrbooking' ),
				'early_cancel_booking'           => esc_html__( 'Early Cancel Booking', 'cbxrbooking' ),
				'early_cancel_desc'              => esc_html__( 'Select how early customers can cancel their booking.',
					'cbxrbooking' ),
				'cancel_admin_alert'             => esc_html__( 'Cancel Request Admin Alert', 'cbxrbooking' ),
				'email_alert_status'             => esc_html__( 'Email Alert Status', 'cbxrbooking' ),
				'cancel_admin_alert_status_desc' => esc_html__( 'Enable/Disable capability if guest can cancel booking',
					'cbxrbooking' ),
				'cancel_user_alert'              => esc_html__( 'Cancel Approve User Alert', 'cbxrbooking' ),
				'title'                          => esc_html__( 'Title', 'cbxrbooking' ),
				'branch'                         => esc_html__( 'Branch', 'cbxrbooking' ),
				'select_branch'                  => esc_html__( 'Select Branch', 'cbxrbooking' ),
				'publish'                        => esc_html__( 'Publish', 'cbxrbooking' ),
				'enabled'                        => esc_html__( 'Enabled', 'cbxrbooking' ),
				'select_form'                    => esc_html__( 'Select form', 'cbxrbooking' ),
				'booking_statistics'             => esc_html__( 'Booking statistics', 'cbxrbooking' ),
				'current_booking_count'          => esc_html__( 'Current booking count', 'cbxrbooking' ),
				'reset_count'                    => esc_html__( 'Reset count', 'cbxrbooking' ),
				// ''                     => esc_html__( '', 'cbxrbooking' ),
			],
			'exportnonce'    => wp_create_nonce( 'cbxrb_export' ),
			'form_statuses'  => self::form_statuses(),
			'default_states' => CBXRBookingHelper::get_default_states(),
			'date_formats'   => CBXRBookingHelper::getAllDateFormat(),
			'early_bookings' => CBXRBookingHelper::get_early_bookings(),
			'late_bookings'  => CBXRBookingHelper::get_late_bookings(),
			'time_interval'  => CBXRBookingHelper::get_time_interval(),
			'week_starts_on' => CBXRBookingHelper::get_week_starts_on(),
			'early_cancels'  => CBXRBookingHelper::get_early_cancels(),
			'admin_email'    => get_bloginfo( 'admin_email' ),
			'form_sections'  => $form_sections,
			'branch_list'    => $branch_list,
			'form_count'     => self::getFormCount()
		];

		$js_translations = array_merge_recursive( $common_js_translations, $form_js_translations );

		return apply_filters( 'cbxrbooking_log_js_translation', $js_translations );
	} //end method cbxrbooking_form_builder_js_translation

	/**
	 * Tools js translation list
	 *
	 * @param $current_user
	 * @param $blog_id
	 *
	 * @return mixed|void
	 */
	public static function cbxrbooking_tools_js_translation() {

		$common_js_translations = self::common_js_translation();

		$tools_js_translations = [
			'translations' => [
				'tools' => [
					'following_option_values' => esc_html__( 'Following option values created by this plugin(including addon) from WordPress core option table',
						'cbxrbooking' ),
					'check_all'               => esc_html__( 'Check All', 'cbxrbooking' ),
					'uncheck_all'             => esc_html__( 'Uncheck All', 'cbxrbooking' ),
					'option_name'             => esc_html__( 'Option Name', 'cbxrbooking' ),
					'option_id'               => esc_html__( 'Option ID', 'cbxrbooking' ),
					'reset_data'              => esc_html__( 'Reset Data', 'cbxrbooking' ),
					'please_select_one'       => esc_html__( 'Please select at least one option', 'cbxrbooking' ),
					'reset_option_data'       => esc_html__( 'Reset option data', 'cbxrbooking' ),
					'show_hide'               => esc_html__( 'Show/Hide', 'cbxrbooking' ),
					'done'                    => esc_html__( 'Done', 'cbxrbooking' ),
					'need_migrate'            => esc_html__( 'Need to migrate', 'cbxrbooking' ),
					'migration_files'         => esc_html__( 'Migration Files', 'cbxrbooking' ),
					'run_migration'           => esc_html__( 'Run Migration', 'cbxrbooking' ),
					'migration_file_name'     => esc_html__( 'Migration File Name', 'cbxrbooking' ),
					'status'                  => esc_html__( 'Status', 'cbxrbooking' ),
					'heading'                 => esc_html__( 'Tools Manager', 'cbxrbooking' ),
				],
			],
			'option_array' => self::getAllOptionNames(),
		];

		$js_translations = array_merge_recursive( $common_js_translations, $tools_js_translations );

		return apply_filters( 'cbxrbooking_tools_js_translation', $js_translations );
	} //end method cbxrbooking_tools_js_translation

	/**
	 * translation for cbxrbooking type
	 *
	 * @param $current_user
	 * @param $blog_id
	 *
	 * @return mixed|void
	 * @since 1.0.0
	 */
	public static function cbxrbooking_dashboard_js_translation() {
		$common_js_translations = self::common_js_translation();

		$tools_js_translations = [
			'translations'     => [
				'heading'            => htmlspecialchars_decode( esc_html__( 'Restaurant Booking : Dashboard', 'cbxrbooking' ) ),
				'dashboard_overview' => esc_html__( 'Dashboard Overview', 'cbxrbooking' ),
				'quick_information'  => esc_html__( 'Quick information of important components', 'cbxrbooking' ),
				'booking'            => esc_html__( 'Booking', 'cbxrbooking' ),
				'bookings_placed'    => esc_html__( 'bookings placed', 'cbxrbooking' ),
				'latest_bookings'    => esc_html__( 'Latest Bookings', 'cbxrbooking' ),
				'form'               => esc_html__( 'Form', 'cbxrbooking' ),
				'forms_created'      => esc_html__( 'forms created', 'cbxrbooking' ),
				'branch'             => esc_html__( 'Branch', 'cbxrbooking' ),
				'branches_created'   => esc_html__( 'branches created', 'cbxrbooking' ),
				'code'               => esc_html__( 'Code', 'cbxrbooking' ),
				'name'               => esc_html__( 'Name', 'cbxrbooking' ),
				'email'              => esc_html__( 'Email', 'cbxrbooking' ),
				'edit'               => esc_html__( 'Edit', 'cbxrbooking' ),

				'booking_submission_overview'         => esc_html__( 'Review Submission Overview', 'cbxrbooking' ),
				'quick_overview_of_monthly'           => esc_html__( 'Quick overview of monthly booking submissions.', 'cbxrbooking' ),
				'monthly_booking_submission_overview' => esc_html__( 'Monthly Review Submission Overview', 'cbxrbooking' ),


			],
			'dashboard_data'   => self::getAdminDashboardData(),
			'booking_statuses' => self::getAllBookingStatus(),
		];

		$js_translations = array_merge_recursive( $common_js_translations, $tools_js_translations );

		return apply_filters( 'cbxrbooking_dashboard_js_translation', $js_translations );
	} //end of method cbxrbooking_dashboard_js_translation

	/**
	 * get job dashboard data
	 *
	 */
	public static function getAdminDashboardData() {
		try {
			$data = [
				'booking_count' => self::getBookingCount(),
				'form_count'    => self::getFormCount(),
				'branch_count'  => self::getBranchCount()
			];

			return $data;
		} catch ( Exception ) {
			return [];
		}
	}//end function getAdminDashboardData

	/**
	 * Translation for table translation
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function table_light_translation() {
		return [
			'loading' => esc_html__( 'Loading...', 'cbxrbooking' ),
			'first'   => esc_html__( 'First', 'cbxrbooking' ),
			'prev'    => esc_html__( 'Prev', 'cbxrbooking' ),
			'next'    => esc_html__( 'Next', 'cbxrbooking' ),
			'last'    => esc_html__( 'Last', 'cbxrbooking' ),
		];
	} //end of method table_light_translation

	/**
	 * get published Froms
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function publishedFroms() {

		$args = [
			'post_type'      => 'cbxrbooking',
			'post_status'    => 'publish',
			'posts_per_page' => - 1,
			'meta_query'     => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				[
					'key'     => '_cbxrbookingformmeta_status',
					'value'   => 1,
					'compare' => '=',
				],
			],
		];

		$posts_list = get_posts( $args );
		$forms      = [];

		foreach ( $posts_list as $post ) {
			$id           = $post->ID;                // Directly access ID
			$title        = $post->post_title;        // Directly access title
			$forms[ $id ] = $title;
		}

		wp_reset_postdata(); // Reset post data if needed elsewhere

		return $forms;
	} //end of method table_light_translation

	/**
	 * Total form count
	 *
	 * @param  int/string $form_id
	 * @param  string  $status
	 * @param  string  $filter_score
	 *
	 * @return int
	 */
	public static function getFormCount() {
		try {
			return Post::bookingForms()->count();
		} catch ( \Exception ) {
			return 0;
		}
	}//end method getFormCount

	/**
	 * Total booking count
	 *
	 * @param  int/string $form_id
	 * @param  string  $status
	 * @param  string  $filter_score
	 *
	 * @return int
	 */
	public static function getBookingCount() {
		try {
			return RBookingLog::query()->count();
		} catch ( \Exception ) {
			return 0;
		}
	}//end method getBookingCount

	/**
	 * Total branch count
	 *
	 * @param  int/string $form_id
	 * @param  string  $status
	 * @param  string  $filter_score
	 *
	 * @return int
	 */
	public static function getBranchCount() {
		try {
			return RBranch::query()->count();
		} catch ( \Exception ) {
			return 0;
		}
	}//end method getBranchCount

	/**
	 * dashboard Bookings array
	 *
	 * @return mixed
	 */
	public static function dashboardBookings() {
		$logs = RBookingLog::query()->orderBy( 'id', 'DESC' )->take( 10 );

		return $logs->get()->toArray();
	}//end method dashboardBookings

	/**
	 * get daily booking count
	 *
	 */
	public static function getDailyBookingCounts( $year = null, $month = null, $status = [] ): array {
		$month = $month ? intval( $month ) : gmdate( 'm' );
		$year  = $year ? intval( $year ) : gmdate( 'Y' );

		// Initialize an array with days of the month, defaulting to 0 income
		$days_in_month = cal_days_in_month( CAL_GREGORIAN, $month, $year );
		$daily_totals  = array_fill( 1, $days_in_month, 0 );

		try {
			if ( $year > 0 && $month > 0 ) {

				// Query the RBookingLog model for the specific month and year
				$query = RBookingLog::selectRaw( 'DAY(add_date) as day, COUNT(*) as count' )
				                    ->whereYear( 'add_date', $year )
				                    ->whereMonth( 'add_date', $month )
				                    ->groupByRaw( 'DAY(add_date)' );

				if ( is_array( $status ) && count( $status ) ) {
					$query->whereIn( 'status', $status );
				}
				// Execute the query
				$results = $query->get();

				// Populate the daily_totals array with the results
				foreach ( $results as $result ) {
					$day_number                  = intval( $result->day );
					$daily_totals[ $day_number ] = intval( $result->count );
				}
			}
		} catch ( \Exception $e ) {

		}

		return $daily_totals;
	}//end function getDailyBookingCounts

	/**
	 * Global Setting Sections and titles
	 *
	 * @return array
	 */
	public static function get_settings_sections() {
		$settings_sections = [
			[
				'id'    => 'cbxrbooking_global',
				'title' => esc_html__( 'Global Setting', 'cbxrbooking' )
			],
			[
				'id'    => 'cbxrbooking_email_tpl',
				'title' => esc_html__( 'Email Template', 'cbxrbooking' )
			]
		];

		return apply_filters( 'cbxbooking_setting_sections', $settings_sections );
	}//end method get_settings_sections

	/**
	 * Get all setting fields
	 *
	 * @return void
	 */
	public static function get_settings_fields() {
		$gust_login_forms = CBXRBookingHelper::guest_login_forms();

		$settings_builtin_fields =
			[
				'cbxrbooking_global'    => [
					'cbxrbooking_global_heading' => [
						'name'    => 'cbxrbooking_global_heading',
						'label'   => esc_html__( 'Global Settings', 'cbxrbooking' ),
						'type'    => 'heading',
						'default' => '',
					],
					'global_date_format'         => [
						'name'    => 'global_date_format',
						'label'   => esc_html__( 'Date Format', 'cbxrbooking' ),
						'desc'    => esc_html__( 'What will be the date format?. This date format will use in global activities like booking logs, calendar, branch, export etc.',
							'cbxrbooking' ),
						'type'    => 'select',
						'default' => '0',
						'options' => CBXRBookingHelper::getAllDateFormat(),
					],
					'global_time_format'         => [
						'name'    => 'time_format',
						'label'   => esc_html__( 'Time Format', 'cbxrbooking' ),
						'desc'    => esc_html__( 'What will be the hour format? 12 or 24. This time format will use in global activities like booking logs, calendar, branch, export etc.',
							'cbxrbooking' ),
						'type'    => 'select',
						'default' => '24',
						'options' => [
							'12' => esc_html__( '12 Hour', 'cbxrbooking' ),
							'24' => esc_html__( '24 Hour', 'cbxrbooking' ),
						],
					],
					'showform_successful'        => [
						'name'    => 'showform_successful',
						'label'   => esc_html__( 'Form show (after successful submission)', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Show form after successful submission.', 'cbxrbooking' ),
						'type'    => 'checkbox',
						'default' => 'on',
						//'desc_tip' => true,
					],
					'show_credit'                => [
						'name'    => 'show_credit',
						'label'   => esc_html__( 'Show Credit Under Form', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Show Credit Under Form.', 'cbxrbooking' ),
						'type'    => 'radio',
						'options' => [
							'yes' => esc_html__( 'Yes', 'cbxrbooking' ),
							'no'  => esc_html__( 'No', 'cbxrbooking' ),
						],
						'default' => 'yes',
						//'desc_tip' => true,
					],
					/*'show_login_form'            => [
						'name'              => 'show_login_form',
						'label'             => esc_html__( 'Show login form for guest user', 'cbxrbooking' ),
						'desc'              => esc_html__( 'If select yes then show the login form for the guest users on the new job creating',
							'cbxrbooking' ),
						'type'              => 'radio',
						'default'           => 'yes',
						'options'           => [
							'yes' => esc_html__( 'Yes', 'cbxrbooking' ),
							'no'  => esc_html__( 'No', 'cbxrbooking' ),
						],
						'sanitize_callback' => 'sanitize_text_field'
					],*/
					'guest_login_form'           => [
						'name'    => 'guest_login_form',
						'label'   => esc_html__( 'Guest User Login Form', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Default guest user is shown wordpress core login form. Pro addon helps to integrate 3rd party plugins like woocommerce, restrict content pro etc.', 'cbxrbooking' ),
						'type'    => 'select',
						'default' => 'wordpress',
						'options' => $gust_login_forms
					],
					'guest_show_register'        => [
						'name'    => 'guest_show_register',
						'label'   => esc_html__( 'Show Register link to guest', 'cbxrbooking' ),
						'desc'    => esc_html__( 'Show register link to guest, depends on if registration is enabled in wordpress core',
							'cbxrbooking' ),
						'type'    => 'radio',
						'default' => 1,
						'options' => [
							1 => esc_html__( 'Yes', 'cbxrbooking' ),
							0 => esc_html__( 'No', 'cbxrbooking' ),
						],
					],
					'delete_global_config'       => [
						'name'    => 'delete_global_config',
						'label'   => esc_html__( 'On Uninstall delete plugin data', 'cbxrbooking' ),
						'desc'    => __( 'Delete Global Config data and custom table created by this plugin on uninstall. <strong>Please note that this can not be undone and you keep proper backup of full database before using feature.</strong>',
							'cbxrbooking' ),
						'type'    => 'radio',
						'options' => [
							'yes' => esc_html__( 'Yes', 'cbxrbooking' ),
							'no'  => esc_html__( 'No', 'cbxrbooking' ),
						],
						'default' => 'no',
					],

				],
				'cbxrbooking_email_tpl' => [
					'cbxrbooking_email_tpl_heading' => [
						'name'    => 'cbxrbooking_email_tpl_heading',
						'label'   => esc_html__( 'Email Template', 'cbxrbooking' ),
						'type'    => 'heading',
						'default' => '',
					],
					'headerimage'                   => [
						'name'    => 'headerimage',
						'label'   => esc_html__( 'Header Image', 'cbxrbooking' ),
						'type'    => 'file',
						'default' => ''
					],
					'footertext'                    => [
						'name'    => 'footertext',
						'label'   => esc_html__( 'Footer Text', 'cbxrbooking' ),
						'desc'    => wp_kses( __( 'The text to appear at the email footer. Syntax available - <code>{site_title}</code>',
							'cbxrbooking' ), [ 'code' => [] ] ),
						'type'    => 'wysiwyg',
						'default' => '{site_title}'
					],
					'basecolor'                     => [
						'name'    => 'basecolor',
						'label'   => esc_html__( 'Base Color', 'cbxrbooking' ),
						'desc'    => esc_html__( 'The base color of the email.', 'cbxrbooking' ),
						'type'    => 'color',
						'default' => '#557da1',
						//'desc_tip' => true,
					],
					'backgroundcolor'               => [
						'name'    => 'backgroundcolor',
						'label'   => esc_html__( 'Background Color', 'cbxrbooking' ),
						'desc'    => esc_html__( 'The background color of the email.', 'cbxrbooking' ),
						'type'    => 'color',
						'default' => '#f5f5f5',
						//'desc_tip' => true,
					],
					'bodybackgroundcolor'           => [
						'name'    => 'bodybackgroundcolor',
						'label'   => esc_html__( 'Body Background Color', 'cbxrbooking' ),
						'desc'    => esc_html__( 'The background colour of the main body of email.',
							'cbxrbooking' ),
						'type'    => 'color',
						'default' => '#fdfdfd',
						//'desc_tip' => true,
					],
					'headingcolor'                  => [
						'name'    => 'headingcolor',
						'label'   => esc_html__( 'Email Heading Color', 'cbxrbooking' ),
						'type'    => 'color',
						'default' => '#ffffff',
						//'desc_tip' => true,
					],
					'bodytextcolor'                 => [
						'name'    => 'bodytextcolor',
						'label'   => esc_html__( 'Body Text Color', 'cbxrbooking' ),
						'desc'    => esc_html__( 'The body text colour of the main body of email.',
							'cbxrbooking' ),
						'type'    => 'color',
						'default' => '#505050',
						//'desc_tip' => true,
					],
					'footertextcolor'               => [
						'name'    => 'footertextcolor',
						'label'   => esc_html__( 'Footer Text Color', 'cbxrbooking' ),
						'desc'    => esc_html__( 'The footer text colour of the footer of email.', 'cbxrbooking' ),
						'type'    => 'color',
						'default' => '#3c3c3c',
					],
				]
			];

		$settings_fields = []; //final setting array that will be passed to different filters

		$sections = CBXRBookingHelper::get_settings_sections();


		foreach ( $sections as $section ) {
			if ( ! isset( $settings_builtin_fields[ $section['id'] ] ) ) {
				$settings_builtin_fields[ $section['id'] ] = [];
			}
		}

		foreach ( $sections as $section ) {
			$settings_builtin_fields_section_id = $settings_builtin_fields[ $section['id'] ];
			$settings_fields[ $section['id'] ]  = apply_filters( 'cbxbooking_global_' . $section['id'] . '_fields',
				$settings_builtin_fields_section_id );
		}


		$settings_fields = apply_filters( 'cbxbooking_global_fields', $settings_fields ); //final filter if need

		return $settings_fields;
	}//end method get_settings_fields

	public static function cbxphpspreadsheet_active() {
		return defined( 'CBXPHPSPREADSHEET_PLUGIN_NAME' ) && cbxphpspreadsheet_loadable();
	}// end method cbxphpspreadsheet_active

	/**
	 * HTML elements, attributes, and attribute values will occur in your output
	 *
	 * @return array
	 * @since 2.0.0
	 */
	public static function allowedHtmlTags() {
		$allowed_html_tags = [
			'a'      => [
				'href'  => [],
				'title' => [],
				//'class' => array(),
				//'data'  => array(),
				//'rel'   => array(),
			],
			'br'     => [],
			'em'     => [],
			'ul'     => [//'class' => array(),
			],
			'ol'     => [//'class' => array(),
			],
			'li'     => [//'class' => array(),
			],
			'strong' => [],
			'p'      => [
				//'class' => array(),
				//'data'  => array(),
				//'style' => array(),
			],
			'span'   => [
				//					'class' => array(),
				//'style' => array(),
			],
		];

		return apply_filters( 'cbxrbooking_allowed_html_tags', $allowed_html_tags );
	}//end method allowedHtmlTags

	/**
	 * Returns codeboxr news feeds using transient cache
	 *
	 * @return false|mixed|\SimplePie\Item[]|null
	 */
	public static function codeboxr_news_feed() {
		$cache_key   = 'codeboxr_news_feed_cache';
		$cached_feed = get_transient( $cache_key );

		$news = false;

		if ( false === $cached_feed ) {
			include_once ABSPATH . WPINC . '/feed.php'; // Ensure feed functions are available
			$feed = fetch_feed( 'https://codeboxr.com/feed?post_type=post' );

			if ( is_wp_error( $feed ) ) {
				return false; // Return false if there's an error
			}

			$feed->init();

			$feed->set_output_encoding( 'UTF-8' );                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          // this is the encoding parameter, and can be left unchanged in almost every case
			$feed->handle_content_type();                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 // this double-checks the encoding type
			$feed->set_cache_duration( 21600 );                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           // 21,600 seconds is six hours
			$limit  = $feed->get_item_quantity( 10 );                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     // fetches the 18 most recent RSS feed stories
			$items  = $feed->get_items( 0, $limit );
			$blocks = array_slice( $items, 0, 10 );

			$news = [];
			foreach ( $blocks as $block ) {
				$url   = $block->get_permalink();
				$url   = CBXRBookingHelper::url_utmy( esc_url( $url ) );
				$title = $block->get_title();

				$news[] = [ 'url' => $url, 'title' => $title ];
			}

			set_transient( $cache_key, $news, HOUR_IN_SECONDS * 6 ); // Cache for 6 hours
		} else {
			$news = $cached_feed;
		}

		return $news;
	}//end method codeboxr_news_feed

	/**
	 * Login forms
	 *
	 * @return mixed|null
	 * @since 2.0.0
	 */
	public static function guest_login_forms() {
		$forms = [];

		$forms['wordpress'] = esc_html__( 'WordPress Core Login Form', 'cbxrbooking' );
		$forms['none']      = esc_html__( 'Don\'t show login form, show default login url', 'cbxrbooking' );
		$forms['off']       = esc_html__( 'Show nothing!', 'cbxrbooking' );

		return apply_filters( 'cbxrbooking_guest_login_forms', $forms );
	}//end method guest_login_forms
}//end class CBXRBookingHelper