<?php

namespace CBX\RBooking\Models;

use Illuminate\Database\Eloquent\Model as Eloquent;

/**
 * Class BookingForm
 *
 * @since 1.0.0
 */
class Post extends Eloquent {
	protected $guarded = [];

	public $timestamps = false;

	protected $appends = [ 'branch', 'submission' ];

	protected $table = 'posts';

	/**
	 * Get the related postmeta for the booking form.
	 */
	public function postmeta() {
		return $this->hasMany( PostMetaModel::class, 'post_id' );
	}

	/**
	 * Scope a query to only include posts of post_type 'cbxrbooking'.
	 *
	 * @param  \Illuminate\Database\Eloquent\Builder  $query
	 *
	 * @return \Illuminate\Database\Eloquent\Builder
	 */
	public function scopeBookingForms( $query ) {
		$query->where( 'post_status', '!=', 'auto-draft' );

		return $query->where( 'post_type', 'cbxrbooking' );
	}

	/**
	 * get formatted create date
	 *
	 * @return string
	 */
	public function getBranchAttribute() {
		if ( ! isset( $this->attributes['ID'] ) ) {
			return '';
		}

		$branchId = get_post_meta( $this->attributes['ID'], '_cbxrbookingformmeta_branch', true );
		$branchId = intval( $branchId );

		if ( $branchId > 0 ) {
			return RBranch::find( $branchId )->name;
		} else {
			return '';
		}
	}//end method getBranchAttribute

	/**
	 * get formatted create date
	 *
	 * @return string
	 */
	public function getSubmissionAttribute() {
		if ( ! isset( $this->attributes['ID'] ) ) {
			return '';
		}

		$submission_count = intval( get_post_meta( $this->attributes['ID'], '_cbxrbookingmeta_submission_count', true ) );

		return $submission_count;
	}//end method getSubmissionAttribute
}// end class Post
