<?php

namespace CBX\RBooking\Models;

use Illuminate\Database\Eloquent\Model as Eloquent;

class RBookingLog extends Eloquent {

	protected $table = 'cbxrbooking_log_manager';

	protected $appends = [ 'formatted_create_date', 'formatted_update_date' ];


	protected $fillable = [
		'form_id',
		'name',
		'email',
		'phone',
		'booking_date',
		'booking_time',
		'party_size',
		'message',
		'user_ip',
		'add_by',
		'mod_by',
		'add_date',
		'mod_date',
		'status',
		'secret',
		'activation',
		'metadata'
	];

	public $timestamps = false;

	/**
	 * delete the form
	 *
	 * @return bool|null
	 */
	public function delete() {
		$log = $this->toArray();
		do_action( 'cbxrbooking_log_delete_before', $log );

		$delete = parent::delete();
		if ( $delete ) {
			do_action( 'cbxrbooking_log_delete_after', $log );
		} else {
			do_action( 'cbxrbooking_log_delete_failed', $log );
		}

		return $delete;
	}// end method delete


	/**
	 * get formatted create date
	 *
	 * @return string
	 */
	public function getFormattedCreateDateAttribute() {
		if ( ! isset( $this->attributes['id'] ) ) {
			return '';
		}
		if ( ! isset( $this->attributes['add_date'] ) ) {
			return '';
		}

		$format = get_option( 'date_format' ) . ' ' . get_option( 'time_format' );

		return date_i18n( $format, strtotime( $this->attributes['add_date'] ) );
	}//end method getFormattedCreateDateAttribute

	/**
	 * get formatted create date
	 *
	 * @return string
	 */
	public function getFormattedUpdateDateAttribute() {
		if ( ! isset( $this->attributes['id'] ) ) {
			return '';
		}
		if ( ! isset( $this->attributes['mod_date'] ) ) {
			return '';
		}

		$format = get_option( 'date_format' ) . ' ' . get_option( 'time_format' );

		return date_i18n( $format, strtotime( $this->attributes['mod_date'] ) );
	}//end method getFormattedUpdateDateAttribute

	/**
	 * Relation between users table
	 *
	 * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
	 */
	public function modUser() {
		return $this->belongsTo( UserModel::class, "mod_by", "ID" );
	}
}//end class RBookingLog