<?php

namespace CBX\RBooking\Models;

use Illuminate\Database\Eloquent\Model as Eloquent;

class RBranch extends Eloquent {

	protected $table = 'cbxrbooking_branch_manager';

	protected $appends = [ 'formatted_create_date', 'formatted_update_date' ];


	protected $fillable = [
		'name',
		'description',
		'address',
		'add_by',
		'mod_by',
		'add_date',
		'mod_date'
	];

	public $timestamps = false;

	/**
	 * delete the form
	 *
	 * @return bool|null
	 */
	public function delete() {
		$manager = $this->toArray();
		do_action( 'cbxrbooking_manager_delete_before', $manager );

		$delete = parent::delete();
		if ( $delete ) {
			do_action( 'cbxrbooking_manager_delete_after', $manager );
		} else {
			do_action( 'cbxrbooking_manager_delete_failed', $manager );
		}

		return $delete;
	}// end method delete


	/**
	 * get formatted create date
	 *
	 * @return string
	 */
	public function getFormattedCreateDateAttribute() {
		if ( ! isset( $this->attributes['id'] ) ) {
			return '';
		}
		if ( ! isset( $this->attributes['add_date'] ) ) {
			return '';
		}

		$format = get_option( 'date_format' ) . ' ' . get_option( 'time_format' );

		return date_i18n( $format, strtotime( $this->attributes['add_date'] ) );
	}//end method getFormattedCreateDateAttribute

	/**
	 * get formatted create date
	 *
	 * @return string
	 */
	public function getFormattedUpdateDateAttribute() {
		if ( ! isset( $this->attributes['id'] ) ) {
			return '';
		}
		if ( ! isset( $this->attributes['mod_date'] ) ) {
			return '';
		}

		$format = get_option( 'date_format' ) . ' ' . get_option( 'time_format' );

		return date_i18n( $format, strtotime( $this->attributes['mod_date'] ) );
	}//end method getFormattedUpdateDateAttribute

	/**
	 * Relation between users table
	 *
	 * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
	 */
	public function modUser() {
		return $this->belongsTo( UserModel::class, "mod_by", "ID" );
	}

	/**
	 * Relation between users table
	 *
	 * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
	 */
	public function addUser() {
		return $this->belongsTo( UserModel::class, "add_by", "ID" );
	}

	/**
	 * Get Custom Question data
	 *
	 * @return array
	 */
	public function getAddressAttribute() {
		if ( isset( $this->attributes['address'] ) && ! is_null( $this->attributes['address'] ) ) {
			return unserialize( $this->attributes['address'] );
		} else {
			return [];
		}
	}//end method getAddressAttribute
}//end class RBranch