<?php

namespace CBX\RBooking\Widgets\Block;

use CBX\RBooking\Helpers\CBXRBookingHelper;

/**
 * Class RBookingBlockWidget
 *
 * Restaurant booking form display
 */
class RBookingBlockWidget {
	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;
	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;


	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->plugin_name = CBXRBOOKING_PLUGIN_NAME;
		$this->version     = CBXRBOOKING_PLUGIN_VERSION;

		global $wp_version;

		$this->init_post_block();

	}//end of construct

	/**
	 * Register bookmark posts block
	 */
	public function init_post_block() {

		$js_url_part = CBXRBOOKING_ROOT_URL . 'assets/js/';
		$js_url_path = CBXRBOOKING_ROOT_PATH . 'assets/js/';

		wp_register_script( 'cbxrbooking-booking-form-block', $js_url_part . 'blocks/cbxrbooking-booking-form-block.js',
			[
				'wp-blocks',
				'wp-element',
				'wp-components',
				'wp-editor'
			],
			filemtime( $js_url_path . 'blocks/cbxrbooking-booking-form-block.js' ), true );

		// phpcs:enable

		$booking_forms = [];

		$booking_forms[0] = esc_html__( 'Select Booking Form', 'cbxrbooking' );

		global $post;
		$args = [
			'post_type'      => 'cbxrbooking',
			'orderby'        => 'ID',
			'order'          => 'DESC',
			'post_status'    => 'publish',
			'posts_per_page' => - 1,
			'meta_query'     => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				[
					'key'     => '_cbxrbookingformmeta_status',
					'value'   => 1,
					'compare' => '=',
				],
			],
		];

		$all_posts = get_posts( $args );
		foreach ( $all_posts as $post ):
			CBXRBookingHelper::setup_postdata( $post );
			$form_id         = get_the_ID();
			$form_title      = get_the_title();
			$booking_forms[] = [
				'label' => esc_attr( $form_title ),
				'value' => $form_id,
			];
		endforeach;

		CBXRBookingHelper::wp_reset_postdata();

		$js_vars = apply_filters( 'cbxrbooking_form_block_js_vars',
			[
				'block_title'      => esc_html__( 'Comfort Restaurant Booking', 'cbxrbooking' ),
				'block_category'   => 'cbxrbooking',
				'block_icon'       => 'universal-access-alt',
				'general_settings' => [
					'heading' => esc_html__( 'Block Settings', 'cbxrbooking' ),
					'id'      => esc_html__( 'Form ID', 'cbxrbooking' ),
					'forms'   => $booking_forms
				],
			] );

		wp_localize_script( 'cbxrbooking-booking-form-block', 'cbxrbooking_form_block', $js_vars );

		register_block_type( 'codeboxr/cbxrbooking-booking-form-block',
			[
				'editor_script'   => 'cbxrbooking-booking-form-block',
				'editor_style'    => 'cbxrbooking-block',
				'attributes'      => apply_filters( 'cbxrbooking_form_block_attributes',
					[
						'id' => [
							'type'    => 'integer',
							'default' => 0,
						],
					] ),
				'render_callback' => [ $this, 'render_block' ],
			] );
	}//end init_post_block

	/**
	 * Getenberg server side render for my bookmark post block
	 *
	 * @param $attr
	 *
	 * @return string
	 */
	public function render_block( $attr ) {
		$arr = [];

		$arr['id'] = $id = isset( $attr['id'] ) ? absint( $attr['id'] ) : 0;

		$attr_html = '';
		foreach ( $arr as $key => $value ) {
			$attr_html .= ' ' . $key . '="' . esc_attr( $value ) . '" ';
		}

		if ( $id > 0 ) {
			return '[cbxrbooking scope="shortcode" ' . $attr_html . ']';
		}
		else{
			return '<p class="cbxrbooking_widget_info">'.esc_html__( 'Please choose booking form', 'cbxrbooking' ).'</p>';
		}
	}//end render_block
}//end method CBXRBookingBlockWidget