<?php

namespace CBX\RBooking\Widgets\Elementor;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use CBX\RBooking\CBXRBookingSettings;
use CBX\RBooking\Helpers\CBXRBookingHelper;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Class RBookingElementorWidget
 *
 * Restaurant booking form display
 */
class RBookingElementorWidget extends Widget_Base {

	/**
	 * Retrieve google maps widget name.
	 *
	 * @return string Widget name.
	 * @since  1.0.0
	 * @access public
	 *
	 */
	public function get_name() {
		return 'cbxrbooking_elemwidget';
	}//end method get_name

	/**
	 * Retrieve google maps widget title.
	 *
	 * @return string Widget title.
	 * @since  1.0.0
	 * @access public
	 *
	 */
	public function get_title() {
		return esc_html__( 'Comfort Restaurant Booking', 'cbxrbooking' );
	}//end method get_title

	/**
	 * Get widget categories.
	 *
	 * Retrieve the widget categories.
	 *
	 * @return array Widget categories.
	 * @since 1.0.10
	 * @access public
	 *
	 */
	public function get_categories() {
		return [ 'cbxrbooking' ];
	}//end method get_categories

	/**
	 * Retrieve google maps widget icon.
	 *
	 * @return string Widget icon.
	 * @since  1.0.0
	 * @access public
	 *
	 */
	public function get_icon() {
		return 'cbxrbooking-icon';
	}//end method get_icon

	/**
	 * Register google maps widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since  1.0.0
	 * @access protected
	 */
	protected function _register_controls() {
		$this->start_controls_section(
			'section_cbxrbooking',
			[
				'label' => esc_html__( 'Booking Form', 'cbxrbooking' ),
			]
		);

		$booking_forms = [];

		$booking_forms[0] = esc_html__( 'Select Booking Form', 'cbxrbooking' );

		global $post;
		$args = [
			'post_type'      => 'cbxrbooking',
			'orderby'        => 'ID',
			'order'          => 'DESC',
			'post_status'    => 'publish',
			'posts_per_page' => - 1,
			'meta_query'     => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				[
					'key'     => '_cbxrbookingformmeta_status',
					'value'   => 1,
					'compare' => '=',
				],
			],
		];

		$all_posts = get_posts( $args );
		foreach ( $all_posts as $post ):
			CBXRBookingHelper::setup_postdata( $post );
			$form_id                   = get_the_ID();
			$form_title                = get_the_title();
			$booking_forms[ $form_id ] = esc_attr( $form_title );
		endforeach;
		CBXRBookingHelper::wp_reset_postdata();


		$this->add_control(
			'cbxrbooking_post_id',
			[
				'label'       => esc_html__( 'Select Booking Form', 'cbxrbooking' ),
				'type'        => Controls_Manager::SELECT,
				'placeholder' => esc_html__( 'Select Booking Form', 'cbxrbooking' ),
				'options'     => $booking_forms,
				'default'     => 0,
			]
		);

		$this->end_controls_section();
	}//end method _register_controls

	/**
	 * Render google maps widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since  1.0.0
	 * @access protected
	 */
	protected function render() {
		$attr = new CBXRBookingSettings();

		$attr = $this->get_settings();
		$id       = intval( $attr['cbxrbooking_post_id'] );
		if ( $id > 0 ) {
			echo do_shortcode( '[cbxrbooking scope="shortcode" id="' . $id . '"]' );
		} else {
			echo '<p class="cbxrbooking_widget_info">'.esc_html__( 'Please choose booking form', 'cbxrbooking' ).'</p>';
		}
	}//end render
}//end class RBookingElementorWidget
