<?php

use Illuminate\Database\Capsule\Manager as Capsule;


if ( ! class_exists( 'CreateCBXBookingBranchManagerTable' ) ) {
	/**
	 * Common migration class for migration table and other tables(codeboxr's plugin or 3rd party if anyone use)
	 *
	 * Class CreateCBXBookingBranchManagerTable
	 * @since 1.0.0
	 */
	class CreateCBXBookingBranchManagerTable {

		/**
		 * Run migrations
		 *
		 * @since 1.0.0
		 */
		public static function up() {
			//migrations table create if not exists
			try {
				if ( ! Capsule::schema()->hasTable( 'cbxrbooking_branch_manager' ) ) {
					Capsule::schema()->create( 'cbxrbooking_branch_manager', function ( $table ) {
						$table->increments( 'id' );
						$table->string( 'name' )->comment('branch manager name');
						$table->string( 'description' )->nullable()->comment('branch manager description');
						$table->text( 'address' )->nullable()->comment('branch manager address');
						$table->integer( 'add_by' )->default(0)->comment('foreign key of user table. who add this branch manager');
						$table->integer( 'mod_by' )->default(0)->comment('foreign key of user table. who last modify this branch manager');
						$table->datetime( 'add_date' )->comment('add date');
						$table->datetime( 'mod_date' )->nullable()->comment('last modified date');

					} );
				}
			} catch ( \Exception $e ) {
				if ( function_exists( 'write_log' ) ) {
					write_log( $e->getMessage() );
				}
			}
		}//end method up

		/**
		 * Migration drop
		 *
		 * @return true|void
		 */
		public static function down() {
			try {
				if ( Capsule::schema()->hasTable( 'cbxrbooking_branch_manager' ) ) {
					return true;
				}
			} catch ( \Exception $e ) {
				if ( function_exists( 'write_log' ) ) {
					write_log( $e->getMessage() );
				}
			}
		}//end method down

	}//end class CreateCBXBookingBranchManagerTable
}


if ( isset( $action ) && $action == 'up' ) {
	CreateCBXBookingBranchManagerTable::up();
} elseif ( isset( $action ) && $action == 'drop' ) {
	CreateCBXBookingBranchManagerTable::down();
}
