<?php

use Illuminate\Database\Capsule\Manager as Capsule;


if ( ! class_exists( 'CreateCBXBookingLogManagerTable' ) ) {
	/**
	 * Common migration class for migration table and other tables(codeboxr's plugin or 3rd party if anyone use)
	 *
	 * Class CreateCBXBookingLogManagerTable
	 * @since 1.0.0
	 */
	class CreateCBXBookingLogManagerTable {

		/**
		 * Run migrations
		 *
		 * @since 1.0.0
		 */
		public static function up() {
			//migrations table create if not exists
			try {
				if ( ! Capsule::schema()->hasTable( 'cbxrbooking_log_manager' ) ) {
					Capsule::schema()->create( 'cbxrbooking_log_manager', function ( $table ) {
						$table->increments( 'id' );
						$table->integer( 'form_id' )->comment('booking form id of custom post type');
						$table->string( 'name' )->comment('client name');
						$table->string( 'email' , 100 )->comment('client email');
						$table->string( 'phone' , 25 )->comment('client phone number');
						$table->date( 'booking_date' )->comment('client preferred booking date');
						$table->time( 'booking_time' )->comment('client preferred booking time');
						$table->integer( 'party_size' )->default(0)->comment('client preferred party size');
						$table->text( 'message' )->nullable()->comment('Client Message');
						$table->string( 'user_ip' , 45 );
						$table->integer( 'add_by' )->default(0)->comment('foreign key of user table. who added this, if set to zero');
						$table->integer( 'mod_by' )->default(0)->comment('foreign key of user table. who last modify this list');
						$table->datetime( 'add_date' )->comment('add date');
						$table->datetime( 'mod_date' )->nullable()->comment('last modified date');
						$table->string( 'status' )->default('pending');
						$table->string( 'secret' )->nullable()->comment('booking secret code');
						$table->string( 'activation' )->nullable()->comment('activation code');
						$table->text( 'metadata' )->nullable()->comment('any extra information needed information');
					} );
				}
			} catch ( \Exception $e ) {
				if ( function_exists( 'write_log' ) ) {
					write_log( $e->getMessage() );
				}
			}
		}//end method up

		/**
		 * Migration drop
		 *
		 * @return true|void
		 */
		public static function down() {
			try {
				if ( Capsule::schema()->hasTable( 'cbxrbooking_log_manager' ) ) {
					return true;
				}
			} catch ( \Exception $e ) {
				if ( function_exists( 'write_log' ) ) {
					write_log( $e->getMessage() );
				}
			}
		}//end method down

	}//end class CreateCBXBookingLogManagerTable
}


if ( isset( $action ) && $action == 'up' ) {
	CreateCBXBookingLogManagerTable::up();
} elseif ( isset( $action ) && $action == 'drop' ) {
	CreateCBXBookingLogManagerTable::down();
}
